	program Floppy_Copy
!	Program Floppy_Copy reads an ADCP pingdata file which is stored
!	as an unformatted IBM file and copies the file into an ASCII
!	formatted or unformatted file.  If the formatted ASCII file option
!	is selected, then a choice is made as to whether the file is
!	to be SYS$OUTPUT or if it is to be stored on disk.  One may choose
!	a full or a brief output. 

	IMPLICIT NONE

	call Main
	end !Program Floppy_Copy


	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Main

	IMPLICIT NONE

!	CONSTANTS

	integer*2	NUMBER_OF_BYTES_IN_RECORD
	PARAMETER	( NUMBER_OF_BYTES_IN_RECORD = 128 )
!	NOTE:  IF THE NUMBER_OF_BYTES_IN_RECORD CONSTANT IS ALTERED,	!
!	       THEN CORRESPONDING ADJUSTMENTS MAY HAVE TO BE MADE TO    !
!	       BE MADE TO SOME LINES OF CODE IN SUBROUTINE	        !
!	       DECIDE_IF_HEADER					        !

	integer*2	NUMBER_OF_HEADER_BYTES
	PARAMETER	( NUMBER_OF_HEADER_BYTES = 256 )
!	NOTE:  IF THE NUMBER_OF_HEADER_BYTES CONSTANT IS ALTERED,	!
!	       THEN CORRESPONDING ADJUSTMENTS MAY HAVE TO BE MADE TO    !
!	       BE MADE TO SOME LINES OF CODE IN SUBROUTINE	        !
!	       DECIDE_IF_HEADER					        !

	integer*2	NUMBER_OF_DATA_BYTES
	PARAMETER	( NUMBER_OF_DATA_BYTES = 4992 )

	byte		STRING_LENGTH
	PARAMETER	( STRING_LENGTH = 60 )

	byte		NUMBER_OF_COLUMNS
	PARAMETER	( NUMBER_OF_COLUMNS = 4 )

	integer*2	USER_EXIT_BUF_LENGTH
	PARAMETER	( USER_EXIT_BUF_LENGTH  = 2048  )

	integer*2	NUM_OF_CORRECTIONS
	PARAMETER	( NUM_OF_CORRECTIONS = 20 )

!	LOCAL VARIABLES

	logical*1	The_First_Time_Through / .true. /
	character*80 	ping_file_name
	character*80 	outunit
	logical*1	Binary_Output / .true. /
	logical*1	ASCII_Output
	logical*1	Output_To_Screen
	logical*1	Brief_Output
	integer*4	Hilo	/ 1 /
	character*16	This_String
	integer*4	ASCII_Unit_No / 0 /
	integer*4	User_exit_Unit_No / 0 /
	integer*4	Input_File_Unit_No
	integer*4	Current_Num_Of_Bytes_Processed / 0 /
	integer*4	Last_Num_Of_Bytes_Processed / 0 /
	integer*4	Byte_Counter
	equivalence	( Byte_Counter, Current_Num_Of_Bytes_Processed )
	integer*2	Loop_Count / 0 /
	character*80	Binary_File_String
	character*80	ASCII_File_String
	real*4 		rdummy
	integer*4	iunit
	logical*1	Implement_Sound_Correction
	logical*1	Implement_Time_Correction
	integer*4	End_Condition
	integer*4	Ensemble_Counter / 0 /
	integer*2	First_Ensemble_Processed / 0 /
	real*4		First_Ensemble_Time / 0 /
	integer*2	First_Ensemble_Year / 0 /
	structure	/ Time_Corrections_Record /
	   real*4	Start_Time
	   real*4	Stop_Time
	   real*8	Intercept
	   real*8	Slope
	end structure
	record /Time_Corrections_Record/ Time_Corrections( NUM_OF_CORRECTIONS )
	integer*2	ntc
	real*4 		rck4
	real*4		Sound_Vel_Factor
	real*4		std_dev_temp
	real*4		std_dev_heading
	real*4		std_dev_pitch
	real*4		std_dev_roll
	real*4		Julian_Date
	real*4		Grejul	
	logical*1	New_Block_Is_Data
	logical*1	File_Is_Open
	integer*4	lbf,l1,l2
c
	include 'adcp_var.include /list'
	include 'output_var.include /list'
	include 'ue4_var.include /list'

	call Prepare_Files( Number_Of_Bytes_In_Record,
     :			    Last_Num_Of_Bytes_Processed, Ping_File_Name,
     :			    Input_File_Unit_No, Binary_Output,
     :			    Binary_File_String, iunit,
     :			    Output_To_Screen, Brief_Output,
     :			    ASCII_Output, ASCII_File_String,
     :			    ASCII_Unit_No, First_Ensemble_Processed,
     :			    Ensemble_Number, Ensemble_Counter,
     :			    First_Ensemble_Time, id,
     :			    rd, First_Ensemble_Year,
     :			    Ensemble_Year,
     :                      User_exit_unit_no )

	if ( ( Binary_Output ) .OR. ( ASCII_Output ) ) then
	   call Ask_About_Sound_Vel_Correction( NUMBER_OF_BYTES_IN_RECORD,
     :					        Implement_Sound_Correction,
     :					        Last_Num_Of_Bytes_Processed,
     :						Input_File_Unit_No, iunit,
     :					        ASCII_Output, ASCII_Unit_No,
     :			   		        First_Ensemble_Processed,
     :					        Ensemble_Number,
     :					        Ensemble_Counter,
     :					        First_Ensemble_Time, id, rd,
     :			   		        First_Ensemble_Year,
     :					        Ensemble_Year )

	   call Ask_About_IBM_Time_Correction( NUMBER_OF_BYTES_IN_RECORD,
     :					       Implement_Time_Correction,
     :					       Last_Num_Of_Bytes_Processed,
     :					       Input_File_Unit_No, iunit,
     :					       ASCII_Output, ASCII_Unit_No,
     :			   		       First_Ensemble_Processed,
     :					       Ensemble_Number,
     :					       Ensemble_Counter,
     :					       First_Ensemble_Time, id, rd,
     :			   		       First_Ensemble_Year,
     :					       Ensemble_Year )
	   if ( Implement_Time_Correction )
     :	     call Fetch_Time_Corrections_Data( NUM_OF_CORRECTIONS, ntc,
     :					       Time_Corrections )
	end if

	call Inform_User_Or_Exit_Program( ASCII_Output, Binary_Output,
     :					  Binary_File_String,
     :					  ASCII_File_String,
     :					  Output_To_Screen )
!	That is, if output is not directed to the screen, then the user
!	is informed that the program is outputting information to the
!	appropriate files.

5	call Decide_If_Header( NUMBER_OF_BYTES_IN_RECORD,
     :			       NUMBER_OF_HEADER_BYTES,
     :			       NUMBER_OF_DATA_BYTES,
     :			       Last_Num_Of_Bytes_Processed, iunit,
     :			       Input_File_Unit_No, ASCII_Output,
     :			       ASCII_Unit_No, Line, Is_Header, Hdr,
     :			       Data, Num_Data_Rec, Last_Byte,
     :			       The_First_Time_Through,
     :			       First_Ensemble_Processed,
     :			       Ensemble_Number, Ensemble_Counter,
     :			       First_Ensemble_Time, id, rd,
     :			       First_Ensemble_Year, Ensemble_Year )

	call Initialize_Flags( NUMBER_OF_HEADER_BYTES,
     :			       Hdr, Last_Rec_Flag,
     :			       Bot_Track_Params_Rec, Aber_Param_Rec,
     :			       Header_Notes_Rec, L,
     :			       Last_Dat_Flag, Hrpt_Stats_Rec,
     :			       Ship_Vel_Stats_Rec, Loran_Rec,
     :			       Velocity_Data_Rec, Velocity_Stats_Rec,
     :			       Amp_Data_Rec, Percent_Good_Rec,
     :			       Bot_Track_Data_Rec, User_Exit_Buf_Rec,
     :			       Ctd_Data_Rec, Avg_Spec_Width_Rec,
     :			       Raw_Dop_Data_Rec, Raw_Amp_Data_Rec,
     :			       Raw_Spec_Width_Rec, Num_Three_Beam_Rec,
     :			       Beam_Quality_Stats_Rec, Std_Dev_Vel_Rec )

	call Assign_Header_Variables( NUMBER_OF_HEADER_BYTES,
     :				      STRING_LENGTH, Last_Dat_Flag,
     :				      Hdr, Trans_Up_Down,
     :				      Trans_Cave_Vex, Pings_Per_Ensemble,
     :				      Number_Of_Bins, Bin_Length,
     :				      Pulse_Length, Blank_Interval,
     :				      Percent_Good_Threshold,
     :				      Das_Year, Das_Month, Das_Day,
     :				      Das_Hour, Das_Minute, Das_Second,
     :				      Das_Hund_Of_Sec, Id_Length,
     :				      Cruise_ID, Velocity_Reference,
     :				      Interface_Type, Bottom_Tracking_On,
     :				      Transformed_Data_On, velx_data_rec,
     :				      vely_data_rec, velz_data_rec,
     :				      velerr_data_rec, roll_compensate_on,
     :				      pitch_compensate_on,
     :				      heading_compensate_on, Roll_Offset,
     :				      Pitch_Offset, Heading_Offset,
     :				      Sampling_Interval,
     :				      First_Bin_Ref_Layer,
     :				      Last_Bin_Ref_Layer,
     :				      Temperature_Scale,
     :				      Temperature_Offset, Salinity,
     :				      Depth_Offset, Header_Notes_Rec,
     :				      Header_Notes_Len, Header_Notes )

c	end of header - already read first line of data - get rest

100     read( UNIT=Input_File_Unit_No,end=105, IOSTAT=End_Condition )
     :	( line( Byte_Counter ),Byte_Counter=1, NUMBER_OF_BYTES_IN_RECORD )
	Last_Num_Of_Bytes_Processed = Current_Num_Of_Bytes_Processed - 1
	Loop_Count = Loop_Count + 1
	new_data_rec = ick2(line(1))
	if(new_data_rec .eq. (num_data_rec + 1))then
	call Assign_To_Data_Array( NUMBER_OF_BYTES_IN_RECORD,
     :				   NUMBER_OF_DATA_BYTES,
     :			 	   Last_Byte, Data, Line, Num_Data_Rec,
     :			 	   New_Data_Rec )
	goto 100
	else
	Loop_Count = 0

105	IF ( ( ( Current_Num_OF_Bytes_Processed - 1 )
     :  .GT. 0 ) .AND. ( ( Current_Num_OF_Bytes_Processed - 1 )
     :	.LT. NUMBER_OF_BYTES_IN_RECORD ) )
     :		Last_Num_Of_Bytes_Processed
     :		= Current_Num_OF_Bytes_Processed - 1
! That is, if the Current_Num_Of_Bytes_Processed ( counter ) is not
! equal to the NUMBER_OF_BYTES_IN_RECORD + 1 (i.e. a complete record is
! processed), and if the Current_Num_Of_Bytes_Processed is not equal to 1
! (i.e. a normal end of file has been reached after processing
! a complete record), then an end of file has been reached in the middle
! of the record.  In this case the Last_Num_Of_Bytes_Processed is re-evaluated
! to reflect the number of bytes that were input from the incomplete record
! before reaching end of file.
	if ( Last_Num_Of_Bytes_Processed
     :      .EQ. NUMBER_OF_BYTES_IN_RECORD) then
	call Compute_Data_Variables( NUMBER_OF_DATA_BYTES,
     :				     Ensemble_Year, Ensemble_Month,
     :				     Ensemble_Day, Ensemble_Hour,
     :				     Ensemble_Minute, Ensemble_Second,
     :				     Ensemble_Hund_Of_Sec,
     :				     Ensemble_Number, Data,
     :				     Samples_Per_Average, Hrpt_Stats_Rec,
     :				     Avg_Roll, Avg_Pitch, Avg_Heading,
     :				     Avg_Temp, Temperature_Offset,
     :				     Temperature_Scale, Std_Dev_Temp,
     :				     Std_Dev_Heading, Std_Dev_Pitch,
     :				     Std_Dev_Roll, L, ASCII_Unit_No,
     :				     ASCII_Output )
c
	if(ensemble_number.eq.0)goto 21212
c
	call Conditional_Assignments( NUMBER_OF_BYTES_IN_RECORD,
     :				      NUMBER_OF_COLUMNS,
     :				      NUMBER_OF_HEADER_BYTES,
     :				      NUMBER_OF_DATA_BYTES,
     :				      USER_EXIT_BUF_LENGTH,
     :				      Loran_Rec, Velocity_Data_Rec,
     :				      Velx_Data_Rec, Vely_Data_Rec,
     :				      Velz_Data_Rec, Velerr_Data_Rec,
     :				      Number_Of_Bins, Data,
     :				      L, Vel_X,
     :				      Vel_Y, Vel_Z,
     :				      Vel_Err, Velocity_Stats_Rec,
     :				      Amp_Data_Rec, Avg_Amplitude,
     :				      Percent_Good_Rec, Percent_Good,
     :				      Bot_Track_Data_Rec, Ew_Bottom,
     :				      Ns_Bottom, Bottom_Depth,
     :				      Bottom_Sampling_Interval,
     :				      Loran_Latitude, Loran_Longitude,
     :				      Loran_Velocity, Loran_Direction,
     :				      User_Exit_Buf_Rec, User_Buf_Len,
     :				      Hdr, User_Exit_Buf, Ctd_Data_Rec,
     :				      Avg_Spec_Width_Rec, Avg_Spec_Width,
     :				      Raw_Dop_Data_Rec, Raw_Dop_Val,
     :				      Raw_Amp_Data_Rec, Raw_Amp_Val,
     :				      Raw_Spec_Width_Rec, Raw_Spec_Width,
     :				      Num_Three_Beam_Rec, Num_Three_Beam,
     :				      Beam_Quality_Stats_Rec, Beam_1_Good,
     :				      Beam_2_Good, Beam_3_Good,
     :				      Beam_4_Good, Std_Dev_Vel_Rec,
     :				      Std_Dev_Vel )

	call Output_Init

	call Get_Julian_Time( Ensemble_Year, Ensemble_Month,
     :			      Ensemble_Day, Ensemble_Hour,
     :			      Ensemble_Minute, Ensemble_Second,
     :			      id, rd )

	call Assign_To_Data_Flags( data_flag,
     :				   bottom_tracking_on,
     :				   velx_data_rec,
     :				   vely_data_rec,
     :				   velz_data_rec,
     :				   velerr_data_rec,
     :				   roll_compensate_on,
     :				   pitch_compensate_on,
     :				   heading_compensate_on, 
     :				   hrpt_stats_rec,
     :				   ship_vel_stats_rec,
     :				   loran_rec,
     :				   velocity_data_rec,
     :				   velocity_stats_rec,
     :				   amp_data_rec,
     :				   percent_good_rec,
     :				   bot_track_data_rec,
     :				   user_exit_buf_rec,
     :				   ctd_data_rec,
     :				   avg_spec_width_rec,
     :				   raw_dop_data_rec,
     :				   raw_amp_data_rec,
     :				   raw_spec_width_rec,
     :				   num_three_beam_rec,
     :				   beam_quality_stats_rec,
     :				   std_dev_vel_rec )

	call Make_Corrections_If_Requested( Num_Of_Corrections, id,
     :					    rd, ntc,
     :					    Implement_Sound_Correction,
     :					    Implement_Time_Correction,
     :					    Avg_Temp, Salinity, Rv,
     :					    Sound_Vel_Factor,
     :					    Time_Corrections )

	call Velocity_Assignments( Number_Of_Bytes_In_Record,
     :				   Data_Flag, Number_Of_Bins,
     :				   vx, vel_x, vy, vel_y,
     :				   vz, vel_z, ve, vel_err,
     :				   Sound_Vel_Factor )

	call Establish_First_Ensemble_Info( Ensemble_Counter,
     :					    First_Ensemble_Processed,
     :					    Ensemble_Number,
     :					    First_Ensemble_Time,
     :					    id, rd,
     :					    First_Ensemble_Year,
     :					    Ensemble_Year )

	call  Assign_To_RealX4_Variables( Number_Of_Bytes_In_Record,
     :					  Number_Of_Columns, rw,
     :					  Bin_Length, rx,
     :					  Pulse_Length, rb,
     :					  Blank_Interval, rr,
     :					  Roll_Offset, rp,
     :					  Pitch_Offset, rh,
     :					  Heading_Offset, ro,
     :					  Depth_Offset, rs,
     :					  Sampling_Interval, rf,
     :					  First_Bin_Ref_Layer, rl,
     :					  Last_Bin_Ref_Layer, Data_Flag,
     :					  rat, Avg_Temp, rah,
     :					  Avg_Heading, rap,
     :					  Avg_Pitch, rar,
     :					  Avg_Roll, Number_Of_Bins,
     :					  Amp, Avg_Amplitude,
     :					  Pctgd, Percent_Good,
     :					  re, EW_Bottom, rn,
     :					  NS_Bottom, rz, Bottom_Depth,
     :					  avsw, Avg_Spec_Width,
     :					  rdpv, Raw_Dop_Val,
     :					  ramp, Raw_Amp_Val,
     :					  rspw, Raw_Spec_Width,
     :					  n3b, Num_Three_Beam,
     :					  bmg, Beam_1_Good,
     :					  Beam_2_Good, Beam_3_Good,
     :					  Beam_4_Good, stdv,
     :					  Std_Dev_Vel )

	if ( Binary_Output ) then
	call Output_To_Binary_File( NUMBER_OF_BYTES_IN_RECORD,
     :				    NUMBER_OF_COLUMNS,
     :				    Ensemble_Year, id, Number_Of_Bins,
     :				    Percent_Good_Threshold, Ensemble_Number,
     :				    Samples_Per_Average,
     :				    rd, rw, rx,
     :				    rb, rr, rp,
     :				    rh, rs, rf,
     :				    rl, ro, rv,
     :				    rat, std_dev_temp, rah,
     :				    std_dev_heading, rap,
     :				    std_dev_pitch, rar,
     :				    Std_Dev_Roll, re,
     :				    rn, rz,
     :				    Data_Flag, vx,
     :				    vy, vz, ve,
     :				    amp, pctgd,
     :				    iunit, avsw,
     :				    rdpv, ramp,
     :				    rspw, n3b,
     :				    bmg, stdv )
	end if !Binary_Output

	if ( ASCII_Output ) then
	   call Output_To_ASCII_File( NUMBER_OF_BYTES_IN_RECORD,
     :				      NUMBER_OF_COLUMNS,
     :				      Ping_File_Name, ASCII_Unit_No,
     :				      Brief_Output, Ensemble_Year, id,
     :				      Number_Of_Bins, Percent_Good_Threshold,
     :				      Ensemble_Number, Samples_Per_Average,
     :				      rd, Bin_Length, Pulse_Length,
     :				      Blank_Interval, Roll_Offset, Pitch_Offset,
     :				      Heading_Offset, Sampling_Interval,
     :				      First_Bin_Ref_Layer, Last_Bin_Ref_Layer,
     :				      Depth_Offset, Rv,
     :				      Avg_Temp, std_dev_temp, Avg_Heading,
     :				      std_dev_heading, Avg_Pitch,
     :				      std_dev_pitch, Avg_Roll,
     :				      Std_Dev_Roll, Re,
     :				      Rn, Rz,
     :				      Data_Flag, vx,
     :				      vy, vz, ve,
     :				      amp, pctgd,
     :				      avsw,
     :				      rdpv, ramp,
     :				      rspw, n3b,
     :				      bmg, stdv )
	end if
c
	if(user_exit_buf_rec)then
c	
	write(user_exit_unit_no,9543)ensemble_number,id,rd,avg_heading,
     :             -re,-rn
	write(user_exit_unit_no,7200)iver,inumavg,sec_add
	type *,user_lat_1,user_lon_1
	write(user_exit_unit_no,7201)user_pc_time_1,user_gps_time_1,
c     :        rck8(user_lat_1),rck8(user_lon_1),rck4(height_1),
     :        rck8(user_lat_1),rck8(user_lon_1),rdummy,
     :        ichar(dop_1),ichar(nsat_1),ichar(msg_type_1)
	write(user_exit_unit_no,7201)user_pc_time_2,user_gps_time_2,
c     :        rck8(user_lat_2),rck8(user_lon_2),rck4(height_2),
     :        rck8(user_lat_2),rck8(user_lon_2),rdummy,
     :        ichar(dop_2),ichar(nsat_2),ichar(msg_type_2)
	write(user_exit_unit_no,7201)user_pc_time_3,user_gps_time_3,
c     :        rck8(user_lat_3),rck8(user_lon_3),rck4(height_3),
     :        rck8(user_lat_3),rck8(user_lon_3),rdummy,
     :        ichar(dop_3),ichar(nsat_3),ichar(msg_type_3)
	write(user_exit_unit_no,7201)user_pc_time_4,user_gps_time_4,
c     :        rck8(user_lat_4),rck8(user_lon_4),rck4(height_4),
     :        rck8(user_lat_4),rck8(user_lon_4),rdummy,
     :        ichar(dop_4),ichar(nsat_4),ichar(msg_type_4)
	write(user_exit_unit_no,7202)dh_mean,dh_std,dh_min,dh_max
	write(user_exit_unit_no,7202)p_mean,p_std,p_min,p_max
	write(user_exit_unit_no,7202)r_mean,r_std,r_min,r_max
	write(user_exit_unit_no,7202)mrms_mean,mrms_std,mrms_min,mrms_max
	write(user_exit_unit_no,7202)brms_mean,brms_std,brms_min,brms_max
	write(user_exit_unit_no,7203)n_att,n_used,n_reacq,n_mrms,n_brms,
     :        n_outlier
c
7200	format(3i10)
7202	format(4i10)
7203	format(6i10)
7201	format(2i10,2f14.8,f20.1,3i8)
9543	format('*',x,i7,x,i5,x,f10.7,3f12.1)
	end if
c
	end if

	if ( End_Condition .EQ. .false. ) THEN
	   call Assign_To_New_Block_Type( New_Block_Type, Line( 3 ) )
	   if ( New_Block_Is_Data( New_Block_Type ) ) then
		call Initialize_Data_Array( Data, NUMBER_OF_DATA_BYTES )
C		type *,'num dat byt',number_of_data_bytes
		call Assign_First_128_Data_Bytes( NUMBER_OF_BYTES_IN_RECORD,
     :						  NUMBER_OF_DATA_BYTES,
     :						  Data, Line, Last_Byte,
     :						  Num_Data_Rec )
		call Get_Data_Record_Number( Number_Of_Bytes_In_Record,
     :					     Num_Data_Rec, Line )
		goto 100
            else
c             must be a header block
	      goto 5
	    end if
	return
	else
21212	   call Exit_Routine( Number_Of_Bytes_In_Record,
     :			      Last_Num_Of_Bytes_Processed,
     :			      Input_File_Unit_No, iunit,
     :			      ASCII_Output, ASCII_Unit_No, 'MAIN',
     :			      First_Ensemble_Processed, Ensemble_Number,
     :			      Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			      First_Ensemble_Year, Ensemble_Year )
	end if
	end if
999	stop
	end !Main

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Prepare_Files( Number_Of_Bytes_In_Record,
     :				  Last_Num_Of_Bytes_Processed, Ping_File_Name,
     :				  Input_File_Unit_No, Binary_Output,
     :				  Binary_File_String, iunit,
     :				  Output_To_Screen, Brief_Output,
     :				  ASCII_Output, ASCII_File_String,
     :				  ASCII_Unit_No, First_Ensemble_Processed,
     :				  Ensemble_Number, Ensemble_Counter,
     :				  First_Ensemble_Time, id,
     :				  rd, First_Ensemble_Year,
     :				  Ensemble_Year ,
     :                            User_exit_unit_no )
!	Subroutine Prepare_Files prepares the appropriate files.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*4	Last_Num_Of_Bytes_Processed
	character*(*)	Ping_File_Name
	integer*4	Input_File_Unit_No
	logical*1	Binary_Output
	character*(*)	Binary_File_String
	integer*2	iunit
	logical*1	Output_To_Screen
	logical*1	Brief_Output
	logical*1	ASCII_Output
	character*(*)	ASCII_File_String
	integer*4	ASCII_Unit_No
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year
	integer*4       User_exit_unit_no

	call Get_And_Open_Input_File( Ping_File_Name, Input_File_Unit_No )

	call Ask_Which_Output_Files_To_Open ( Number_Of_Bytes_In_Record,
     :					      iunit, Binary_Output,
     :					      Output_To_Screen,
     :					      Brief_Output,
     :					      Binary_File_String,
     :					      ASCII_File_String,
     :					      Last_Num_Of_Bytes_Processed,
     :					      Input_File_Unit_No,
     :					      ASCII_Output, ASCII_Unit_No,
     :					      First_Ensemble_Processed,
     :					      Ensemble_Number,
     :					      Ensemble_Counter,
     :					      First_Ensemble_Time,
     :					      id, rd,
     :					      First_Ensemble_Year,
     :					      Ensemble_Year,
     :                                        User_exit_unit_no )

	end !Prepare_Files

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Get_And_Open_Input_File( Ping_File_Name, Input_File_Unit_No )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	character*(*)	Ping_File_Name
	integer*4	Input_File_Unit_No

!	LOCAL VARIABLES

	character*80	Ping_File_Symbol
	integer*4	Ping_File_Symbol_Length / 0 /

	print *, ' '
	type *,'Enter complete pingdata file name.'
	accept '(a)', Ping_File_Symbol
	call STR$UPCASE( Ping_File_Symbol, Ping_File_Symbol )
	call LIB$GET_SYMBOL( Ping_File_Symbol, Ping_File_Name,
     :				Ping_File_Symbol_Length  )
	if ( Ping_File_Symbol_Length .EQ. 0 )
     :	   Ping_File_Name = Ping_File_Symbol
	call LIB$GET_LUN( Input_File_Unit_No )
	open( UNIT = Input_File_Unit_No, FILE = Ping_File_Name, STATUS = 'OLD',
     :	      READONLY, FORM = 'UNFORMATTED')
	end !Get_And_Open_Input_File

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Ask_Which_Output_Files_To_Open( Number_Of_Bytes_In_Record,
     :						   iunit, Binary_Output,
     :						   Output_To_Screen,
     :						   Brief_Output,
     :						   Binary_File_String,
     :						   ASCII_File_String,
     :						   Last_Num_Of_Bytes_Processed,
     :						   Input_File_Unit_No,
     :						   ASCII_Output, ASCII_Unit_No,
     :						   First_Ensemble_Processed,
     :						   Ensemble_Number,
     :						   Ensemble_Counter,
     :						   First_Ensemble_Time,
     :						   id, rd,
     :						   First_Ensemble_Year,
     :						   Ensemble_Year,
     :                                             User_exit_unit_no )

	IMPLICIT NONE

!	CONSTANTS

	character*1     YES
	PARAMETER	( YES = 'Y' )
	character*1     NO
	PARAMETER	( NO  = 'N' )

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*2	iunit
	logical*1	Binary_Output
	logical*1	Output_To_Screen
	logical*1	Brief_Output
	character*(*)	Binary_File_String
	character*(*)	ASCII_File_String
	integer*4	Last_Num_Of_Bytes_Processed
	integer*4	Input_File_Unit_No
	logical*1	ASCII_Output
	integer*4	ASCII_Unit_No
	integer*4	User_Exit_Unit_No
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*4	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	character*1     reply / 'N' /
	logical*1	Answer_Not_Satisfactory / .true. /

	do while ( Answer_Not_Satisfactory )
	   print *, ' '
	   print 880
880	  format  (' Would you like to write to a binary file? (Y/N) ', $ )

	   read (UNIT = *, FMT = '(A)', END = 51115, ERR = 2020 ) ( Reply ) 
1919	   Format ( I1 )
	   call str$upcase( Reply, Reply )
	   if(reply.eq.yes)then
	      Binary_Output	      = .true.
	      Answer_Not_Satisfactory = .false.
	   else if ( Reply .EQ. no ) then
	      Binary_Output	      = .false.
	      Answer_Not_Satisfactory = .false.
	   else
2020	      Print *, ' '
	      Print *, 'Please enter only "Y" for "yes" or "N" for "no".'
	   end if
	end do
	if ( Binary_Output ) call Get_And_Open_Binary_File( iunit,
     :							Binary_File_String )

	Answer_Not_Satisfactory = .true.

	do while ( Answer_Not_Satisfactory )
	   print *, ' '
	   print *,  'Would you like to write to an ASCII file?'
	   print 990
990	   format ( ' Please remember that the screen is',
     :		    ' considered a file too. (Y/N) ', $ )

	   read (UNIT = *, FMT = '(A)', END = 51115, ERR = 3030 ) ( Reply )
	   call str$upcase( Reply, Reply )
	   if ( Reply .EQ. yes ) then
	      ASCII_Output	      = .true.
	      Answer_Not_Satisfactory = .false.
	   else if ( Reply .EQ. no ) then
	      ASCII_Output 	      = .false.
	      Answer_Not_Satisfactory = .false.
	   else
3030	      Print *, ' '
	   print *, 'Please enter only "Y" for "yes" or "N" for "no".'
	   end if
	end do

	if ( ASCII_Output ) then
	   call Decision_Table( Number_Of_Bytes_In_Record, Brief_Output,
     :				Output_To_Screen,
     :				Last_Num_Of_Bytes_Processed,
     :				Input_File_Unit_No, iunit,
     :				ASCII_Output, ASCII_Unit_No,
     :				First_Ensemble_Processed, Ensemble_Number,
     :				Ensemble_Counter, First_Ensemble_Time,
     :				id, rd, First_Ensemble_Year, Ensemble_Year )
	   call LIB$GET_LUN( ASCII_Unit_No )
	   call open_Ascii_File( ASCII_File_String, ASCII_Unit_No,
     :				 Output_To_Screen )
	end if
c
	   call LIB$GET_LUN(User_Exit_Unit_No)
	   open( UNIT = User_exit_Unit_No, FILE = 'USER_EXIT.DAT',
     : STATUS = 'NEW', FORM = 'FORMATTED', recl=2048 )
c
	return
51115	call Exit_Routine( Number_Of_Bytes_In_Record,
     :			   Last_Num_Of_Bytes_Processed,
     :			   Input_File_Unit_No, iunit,
     :			   ASCII_Output, ASCII_Unit_No,
     :			   'ASK_WHICH_OUTPUT_FILES_TO_OPEN',
     :			   First_Ensemble_Processed, Ensemble_Number,
     :			   Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			   First_Ensemble_Year, Ensemble_Year )
	end ! Ask_Which_Output_Files_To_Open

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Decision_Table( Number_Of_Bytes_In_Record, Brief_Output,
     :				   Output_To_Screen,
     :				   Last_Num_Of_Bytes_Processed,
     :				   Input_File_Unit_No, iunit,
     :				   ASCII_Output, ASCII_Unit_No,
     :				   First_Ensemble_Processed, Ensemble_Number,
     :				   Ensemble_Counter, First_Ensemble_Time,
     :				   id, rd, First_Ensemble_Year, Ensemble_Year )
!	Subroutine Decision_Table presents the user with formatted ASCII
!	options.  Flags are set according to the user's responses.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	logical*1	Brief_Output
	logical*1	Output_To_Screen
	integer*4	Last_Num_Of_Bytes_Processed
	integer*4	Input_File_Unit_No
	integer*4	iunit
	logical*1	ASCII_Output
	integer*4	ASCII_Unit_No
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	integer*2	Decision
	logical*1	Not_Decided / .true. /

	do while ( Not_Decided )
	   call Output_Decision_Table
	   read ( UNIT = *, FMT = '(i)', END = 61116, ERR = 4040 ), Decision
	   if ( Decision .EQ. 1 ) then
	      Not_Decided      = .false.
	      Brief_Output     = .false.
	      Output_To_Screen = .true.
	   else if ( Decision .EQ. 2 ) then
	      Not_Decided      = .false.
	      Brief_Output     = .true.
	      Output_To_Screen = .true.
	   else if ( Decision .EQ. 3 ) then
	      Not_Decided      = .false.
	      Brief_Output     = .false.
	      Output_To_Screen = .false.
	   else if ( Decision .EQ. 4 ) then
	      Not_Decided      = .false.
	      Brief_Output     = .true.
	      Output_To_Screen = .false.
	   else
4040	      Not_Decided =.true.
	      print *, ' '
	      print *, 'Please enter only 1, 2, 3, or 4.'
	   end if	
	end do
	type *, ' '
	type *, ' '
	return
61116	call Exit_Routine( Number_Of_Bytes_In_Record,
     :			   Last_Num_Of_Bytes_Processed,
     :			   Input_File_Unit_No, iunit,
     :			   ASCII_Output, ASCII_Unit_No,
     :			   'DECISION_TABLE',
     :			   First_Ensemble_Processed, Ensemble_Number,
     :			   Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			   First_Ensemble_Year, Ensemble_Year )
	end !Decision_Table

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Output_Decision_Table

	print *, ' '
	print *, '______________________________'
	print *, '|             |              |'
	print *, '| Full Output | Brief Output |'
	print *, '|  to Screen  |  to Screen   |'
	print *, '|             |              |'
	print *, '|   PRESS 1   |   PRESS 2    |'
	print *, '|_____________|______________|'
	print *, '|             |              |'
	print *, '| Full Output | Brief Output |'
	print *, '|  to File    |  to File     |'
	print *, '|             |              |'
	print *, '|   PRESS 3   |   PRESS 4    |'
	print *, '|_____________|______________|'
	print *, ' '
	end !Output_Decision_Table


	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Get_And_Open_Binary_File( iunit, Binary_File_Name )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	character*(*)	Binary_File_Name
	integer*2	iunit

!	LOCAL VARIABLES

	character*80	Binary_File_Symbol
	integer*4	Binary_File_Symbol_Length / 0 /

	call LIB$GET_LUN( iunit )
	type *,'Enter complete binary file name.'
	accept '(a)', Binary_File_Symbol
	call str$upcase( Binary_File_Symbol, Binary_File_Symbol )
	call LIB$GET_SYMBOL( Binary_File_Symbol, Binary_File_Name,
     :				Binary_File_Symbol_Length  )
	if ( Binary_File_Symbol_Length .EQ. 0 )
     :	   Binary_File_Name = Binary_File_Symbol
	    open( unit = iunit, file = Binary_File_Name, status = 'new',
     :		  form = 'unformatted' )
	end !Get_And_Open_Binary_File

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Ask_About_Sound_Vel_Correction( Number_of_Bytes_In_Record,
     :						   Implement_Sound_Correction,
     :						   Last_Num_Of_Bytes_Processed,
     :						   Input_File_Unit_No, iunit,
     :						   ASCII_Output, ASCII_Unit_No,
     :			   			   First_Ensemble_Processed,
     :						   Ensemble_Number,
     :						   Ensemble_Counter,
     :						   First_Ensemble_Time, id, rd,
     :			   			   First_Ensemble_Year,
     :						   Ensemble_Year )
!	Subroutine Ask_About_Sound_Vel_Correction inquires if the user wishes
!	to implement sound velocity corrections and then sets flags
!	accordingly.

	IMPLICIT NONE

!	CONSTANTS

	character	Yes
	PARAMETER( Yes = 'Y' )
	character	No
	PARAMETER( No  = 'N' )

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	logical*1	Implement_Sound_Correction
	integer*4	Last_Num_Of_Bytes_Processed
	integer*4	Input_File_Unit_No
	integer*4	iunit
	logical*1	ASCII_Output
	integer*4	ASCII_Unit_No
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	logical*1	Answer_Not_Satisfactory / .true. /
	character	Reply

	do while ( Answer_Not_Satisfactory )
	   print *, ' '
	   print *,  'Do you wish to correct for sound velocity based on the',
     :			' temperature sensor?'
	   print 1000
1000	   format ( ' If not, then sound velocity defaults to',
     :		    ' 1536.0 cm/sec. ', $ )
	   read (UNIT = *, FMT = '(A)', END = 62226, ERR = 2222 ) ( Reply )
	   call str$upcase( Reply, Reply )
	   if ( Reply .EQ. yes ) then
	      Implement_Sound_Correction = .true.
	      Answer_Not_Satisfactory    = .false.
	   else if ( Reply .EQ. no ) then
	      Implement_Sound_Correction = .false.
	      Answer_Not_Satisfactory    = .false.
	   else
2222	      Print *, ' '
	   print *, 'Please enter only "Y" for "yes" or "N" for "no".'
	   end if
	end do
	return
62226	call Exit_Routine( Number_Of_Bytes_In_Record,
     :			   Last_Num_Of_Bytes_Processed,
     :			   Input_File_Unit_No, iunit,
     :			   ASCII_Output, ASCII_Unit_No,
     :			   'ASK_ABOUT_SOUND_VEL_CORRECTION',
     :			   First_Ensemble_Processed, Ensemble_Number,
     :			   Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			   First_Ensemble_Year, Ensemble_Year )
	end !Ask_About_Sound_Vel_Correction

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Ask_About_IBM_Time_Correction( Number_of_Bytes_In_Record,
     :						  Implement_Time_Correction,
     :						  Last_Num_Of_Bytes_Processed,
     :						  Input_File_Unit_No, iunit,
     :						  ASCII_Output, ASCII_Unit_No,
     :			   			  First_Ensemble_Processed,
     :						  Ensemble_Number,
     :						  Ensemble_Counter,
     :						  First_Ensemble_Time, id, rd,
     :			   			  First_Ensemble_Year,
     :						  Ensemble_Year )
!	Subroutine Ask_About_IBM_Time_Correction asks the user if there
!	are any time corrections to be made.

	IMPLICIT NONE

!	CONSTANTS

	character	Yes
	PARAMETER( Yes = 'Y' )
	character	No
	PARAMETER( No  = 'N' )

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	logical*1	Implement_Time_Correction
	integer*4	Last_Num_Of_Bytes_Processed
	integer*4	Input_File_Unit_No
	integer*4	iunit
	logical*1	ASCII_Output
	integer*4	ASCII_Unit_No
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	logical*1	Answer_Not_Satisfactory / .true. /
	character	Reply

	do while ( Answer_Not_Satisfactory )
	   print *, ' '
	   print *,  'Do you wish to correct IBM times to match GPS?'
	   print 1100
1100	   format ( ' NOTE:  You must know cutoffs and coefficients! ', $ )
	   read (UNIT = *, FMT = '(A)', END = 63336, ERR = 3333 ) ( Reply )
	   call str$upcase( Reply, Reply )
	   if ( Reply .EQ. yes ) then
	      Implement_Time_Correction = .true.
	      Answer_Not_Satisfactory   = .false.
	   else if ( Reply .EQ. no ) then
	      Implement_Time_Correction = .false.
	      Answer_Not_Satisfactory   = .false.
	   else
3333	      Print *, ' '
	   print *, 'Please enter only "Y" for "yes" or "N" for "no".'
	   end if
	end do
	return
63336	call Exit_Routine( Number_Of_Bytes_In_Record,
     :			   Last_Num_Of_Bytes_Processed,
     :			   Input_File_Unit_No, iunit,
     :			   ASCII_Output, ASCII_Unit_No,
     :			   'ASK_ABOUT_IBM_TIME_CORRECTION',
     :			   First_Ensemble_Processed, Ensemble_Number,
     :			   Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			   First_Ensemble_Year, Ensemble_Year )
	end ! Ask_About_IBM_Time_Correction

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Fetch_Time_Corrections_Data( Num_Of_Corrections,
     :						ntc, Time_Corrections )
!	Subroutine Fetch_Time_Corrections_Data requests the time correction
!	information from the user.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Num_Of_Corrections
	integer*2	ntc
	structure	/ Time_Corrections_Record /
	   real*4	Start_Time
	   real*4	Stop_Time
	   real*8	Intercept
	   real*8	Slope
	end structure
	record /Time_Corrections_Record/ Time_Corrections( Num_Of_Corrections )

!	LOCAL VARIABLES

	integer*2	i

	do i=1,20
	   type *,' Entering time corrections data negative times to end entry'
	   type *,' '
	   type *,' Enter : start, stop ibm times for correction ',i
	   accept *,Time_Corrections(i).Start_Time,Time_Corrections(i).Stop_Time
	   if(Time_Corrections(i).Start_Time.lt.0.
     :	       .or. Time_Corrections(i).Stop_Time.lt.0.) then
                  ntc=i-1
	          return
	   end if
	   type *,' Enter intercept and slope for correction ',i
	   accept *, Time_Corrections(i).Intercept, Time_Corrections(i).Slope
	end do
	return
	end !Fetch_Time_Corrections_Data

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Inform_User_Or_Exit_Program( ASCII_Output, Binary_Output,
     :						Binary_File_String,
     :						ASCII_File_String,
     :						Output_To_Screen )
!	Subroutine Inform_User_Or_Exit_Program "sees" which file output
!	options the user selected.  The subroutine then informs the user
!	that the selected files are being written to.  If the user didn't
!	select any file to write to then the program terminates.	

	IMPLICIT NONE

!	FORMAL PARAMETERS

	logical*1	ASCII_Output
	logical*1	Binary_Output
	character*(*)	Binary_File_String
	character*(*)	ASCII_File_String
	logical*1	Output_To_Screen

	print *, ' '
	if ( ASCII_Output .EQ. .false. ) then
	   if ( Binary_Output .EQ. .true. ) then
	      print 4444, 'binary', Binary_File_String
	   else !Binary_Output is false
	      call EXIT
	   end if
	else !ASCII_Output is true
	   if ( Output_To_Screen .EQ. .false. ) then
	      if ( Binary_Output .EQ. .true. ) then
		 print *, 'Writing to files'
	      else !Binary_Output is false
	    	 print 4444, 'ASCII', ASCII_File_String
	      end if
	   end if
	end if
4444	      format  (' Writing to ', a, ' file ', a )
	end !Inform_User_Or_Exit_Program

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Decide_If_Header( Number_Of_Bytes_In_Record,
     :				     Number_Of_Header_Bytes,
     :				     Number_Of_Data_Bytes,
     :				     Last_Num_Of_Bytes_Processed, iunit,
     :				     Input_File_Unit_No, ASCII_Output,
     :				     ASCII_Unit_No, Line, Is_Header, Hdr,
     :				     Data, Num_Data_Rec, Last_Byte,
     :				     The_First_Time_Through,
     :				     First_Ensemble_Processed,
     :				     Ensemble_Number, Ensemble_Counter,
     :				     First_Ensemble_Time, id, rd,
     :				     First_Ensemble_Year, Ensemble_Year )
!	Subroutine Decide_If_Header begins inputting information and decides
!	if the first record is a pingfile header record.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*2	Number_Of_Header_Bytes
	integer*2	Number_Of_Data_Bytes
	integer*4	Last_Num_Of_Bytes_Processed
	byte		Line( Number_Of_Bytes_In_Record )
	logical*1	Is_Header
	byte		Hdr( Number_Of_Header_Bytes )
	byte		Data( Number_Of_Data_Bytes )
	integer*4	Num_Data_Rec
	integer*4	Last_Byte
	logical*1	The_First_Time_Through
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	ASCII_Unit_No
	integer*4	Input_File_Unit_No
	integer*4	iunit
	logical*1	ASCII_Output
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	integer*4	i

!	FUNCTION DECLARATIONS

	integer*4	ick2

	if ( The_First_Time_Through ) then
	   read( UNIT=Input_File_Unit_No, ERR=71117 )
     :		( line( i ), i=1, Number_Of_Bytes_In_Record )
	   The_First_Time_Through = .false.
	end if

	if( ick2( line( 3 ) ) .eq. 'FFFF'X .and. line( 5 ) .eq. 1 ) then
		is_header = .true.
		do i=1, Number_Of_Bytes_In_Record
			hdr( i ) = line( i )
		end do
		read( UNIT = Input_File_Unit_No, END=71117 )
     :			( line( i ), i=1, Number_Of_Bytes_In_Record )
		if ( ick2( line( 1 ) ) .eq. ( ick2( hdr( 1 ) )+1 ) ) then
			do i=3, Number_Of_Bytes_In_Record
			  	hdr( i + ( Number_Of_Bytes_In_Record - 2 ) )
     :					=line( i )
			end do
			read( UNIT = Input_File_Unit_No, END = 71117 )
     :				( line( i ), i = 1, Number_Of_Bytes_In_Record )
			  if ( ick2( line( 1 ) ) .eq.
     :				( ick2( hdr( 1 ) ) + 2 ) ) then
			    do i = 3, Number_Of_Bytes_In_Record
			  	hdr( i + ( Number_Of_Header_Bytes - 4 ) )
     :					= line( i )
			    end do
			  else
			    do i = 1, Number_Of_Bytes_In_Record
				data( i ) = line( i )
			    end do
		   	    num_data_rec = ick2( line( 1 ) )
	  		    last_byte = Number_Of_Bytes_In_Record
			  end if
		else
		  do i = 1, Number_Of_Bytes_In_Record
		     data( i ) = line( i )
		  end do
  	          num_data_rec = ick2( line( 1 ) )
		  last_byte = Number_Of_Bytes_In_Record
		end if
	else
	if ( ASCII_Output .EQ. .false. ) call Open_ASCII_File( 'SYS$OUTPUT',
     :							       ASCII_Unit_No,
     :							       .true. )
		write ( ASCII_Unit_No, * ) ( 'First record is not',
     :					     ' a header record' )
71117	call Exit_Routine( Number_Of_Bytes_In_Record,
     :			   Last_Num_Of_Bytes_Processed,
     :			   Input_File_Unit_No, iunit,
     :			   ASCII_Output, ASCII_Unit_No, 'DECIDE_IF_HEADER',
     :			   First_Ensemble_Processed, Ensemble_Number,
     :			   Ensemble_Counter, First_Ensemble_Time, id, rd,
     :			   First_Ensemble_Year, Ensemble_Year )
	end if
	end !Decide_If_Header

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Initialize_Flags( Number_Of_Header_Bytes,
     :				     Hdr, Last_Rec_Flag,
     :				     Bot_Track_Params_Rec, Aber_Param_Rec,
     :				     Header_Notes_Rec, L,
     :				     Last_Dat_Flag, Hrpt_Stats_Rec,
     :				     Ship_Vel_Stats_Rec, Loran_Rec,
     :				     Velocity_Data_Rec, Velocity_Stats_Rec,
     :				     Amp_Data_Rec, Percent_Good_Rec,
     :				     Bot_Track_Data_Rec, User_Exit_Buf_Rec,
     :				     Ctd_Data_Rec, Avg_Spec_Width_Rec,
     :				     Raw_Dop_Data_Rec, Raw_Amp_Data_Rec,
     :				     Raw_Spec_Width_Rec, Num_Three_Beam_Rec,
     :				     Beam_Quality_Stats_Rec, Std_Dev_Vel_Rec )
!	Subroutine Initialize_Flags initializes the data recording flags.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Header_Bytes
	byte		Hdr( Number_Of_Header_Bytes )
	integer*4	Last_Rec_Flag
	logical*1	Bot_Track_Params_Rec
	logical*1	Aber_Param_Rec
	logical*1	Header_Notes_Rec
	integer*4	L
	integer*4	Last_Dat_Flag
	logical*1	Hrpt_Stats_Rec
	logical*1	Ship_Vel_Stats_Rec
	logical*1	Loran_Rec
	logical*1	Velocity_Data_Rec
	logical*1	Velocity_Stats_Rec
	logical*1	Amp_Data_Rec
	logical*1	Percent_Good_Rec
	logical*1	Bot_Track_Data_Rec
	logical*1	User_Exit_Buf_Rec
	logical*1	Ctd_Data_Rec
	logical*1	Avg_Spec_Width_Rec
	logical*1	Raw_Dop_Data_Rec
	logical*1	Raw_Amp_Data_Rec
	logical*1	Raw_Spec_Width_Rec
	logical*1	Num_Three_Beam_Rec
	logical*1	Beam_Quality_Stats_Rec
	logical*1	Std_Dev_Vel_Rec

!	LOCAL VARIABLES

	integer*4	Counter
	logical*1	Condition_1
	integer*4	i

!	FUNCTION DECLARATIONS

	integer*4	Bck1

		Counter=6
		Condition_1 = .false.
		do while ( ( Counter .le. 9 ) .and. ( .not. Condition_1 ) )
		   if( bck1( hdr( Counter ) ) .eq. 0 ) then
			last_rec_flag = Counter
			Condition_1 = .true.
		   else
			Counter = Counter + 1
		   end if
		end do
20		if(last_rec_flag .gt. 6)then
		   do i = 6, last_rec_flag
			if( bck1( hdr( i ) ) .eq. 5 )
     :				bot_track_params_rec=.true.
			if( bck1( hdr( i ) ) .eq. 8 ) aber_param_rec = .true.
			if( bck1( hdr( i ) ) .eq. 4 ) header_notes_rec = .true.
		   end do
		end if
		L = last_rec_flag
		Counter = L + 1
		Condition_1 = .false.
		do while ( ( Counter .le. L + 19 ) .and. ( .not. Condition_1 ) )
		   if( bck1( hdr( Counter ) ) .eq. 0 ) then
			Last_Dat_Flag = Counter
			Condition_1 = .true.
		   else
			Counter = Counter + 1
		   end if
		end do
30		do i = l + 2, last_dat_flag
			if ( bck1( hdr( i ) ) .eq. 8 ) hrpt_stats_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 9 )
     :					ship_vel_stats_rec = .true.
			if ( bck1( hdr( i ) ) .eq .10 ) loran_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 11 )
     :					velocity_data_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 12 )
     :					velocity_stats_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 13 ) amp_data_rec = .true.
			if ( bck1 ( hdr( i ) ) .eq. 14 )
     :					percent_good_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 15 )
     :					bot_track_data_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 16 )
     :					user_exit_buf_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 17 )
     :					ctd_data_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 18 )
     :					avg_spec_width_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 19 )
     :					raw_dop_data_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 20 )
     :					raw_amp_data_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 21 )
     :					raw_spec_width_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 22 )
     :					num_three_beam_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 23 )
     :					beam_quality_stats_rec = .true.
			if ( bck1( hdr( i ) ) .eq. 24 )
     :					std_dev_vel_rec = .true.
		end do
	end !Initialize_Flags

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_Header_Variables( Number_Of_Header_Bytes,
     :					    String_Length, Last_Dat_Flag,
     :					    Hdr, Trans_Up_Down,
     :					    Trans_Cave_Vex, Pings_Per_Ensemble,
     :					    Number_Of_Bins, Bin_Length,
     :					    Pulse_Length, Blank_Interval,
     :					    Percent_Good_Threshold,
     :					    Das_Year, Das_Month, Das_Day,
     :					    Das_Hour, Das_Minute, Das_Second,
     :					    Das_Hund_Of_Sec, Id_Length,
     :					    Cruise_ID, Velocity_Reference,
     :					    Interface_Type, Bottom_Tracking_On,
     :					    Transformed_Data_On, velx_data_rec,
     :					    vely_data_rec, velz_data_rec,
     :					    velerr_data_rec, roll_compensate_on,
     :					    pitch_compensate_on,
     :					    heading_compensate_on, Roll_Offset,
     :					    Pitch_Offset, Heading_Offset,
     :					    Sampling_Interval,
     :					    First_Bin_Ref_Layer,
     :					    Last_Bin_Ref_Layer,
     :					    Temperature_Scale,
     :					    Temperature_Offset, Salinity,
     :					    Depth_Offset, Header_Notes_Rec,
     :					    Header_Notes_Len, Header_Notes )
!	Subroutine Assign_Header_Variables makes varible assignments from data
!	from the header block.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Header_Bytes
	byte		String_Length
	integer*4	Last_Dat_Flag
	byte		Hdr( Number_Of_Header_Bytes )
	integer*2	Trans_Up_Down
	integer*2	Trans_Cave_Vex
	integer*2	Pings_Per_Ensemble
	integer*2	Number_Of_Bins
	integer*2	Bin_Length
	integer*2	Pulse_Length
	integer*2	Blank_Interval
	integer*2	Percent_Good_Threshold
	integer*2	Das_Year
	integer*2	Das_Month
	integer*2	Das_Day
	integer*2	Das_Hour
	integer*2	Das_Minute
	integer*2	Das_Second
	integer*2	Das_Hund_Of_Sec
	integer*2	Id_Length
	character*1	Cruise_ID( String_Length )
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	integer*2	Velocity_Reference
	integer*2	Interface_Type
	logical*1	Bottom_Tracking_On
	logical*1	Transformed_Data_On	! Parameter declarations
	logical*1	velx_data_rec		! for call of Subroutine
	logical*1	vely_data_rec		! Initialize_Header_Flags
	logical*1	velz_data_rec		! from within this subroutine
	logical*1	velerr_data_rec
	logical*1	roll_compensate_on
	logical*1	pitch_compensate_on
	logical*1	heading_compensate_on
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	real*8		Roll_Offset
	real*8		Pitch_Offset
	real*8		Heading_Offset
	real*8		Sampling_Interval
	integer*2	First_Bin_Ref_Layer
	integer*2	Last_Bin_Ref_Layer
	real*8		Temperature_Scale
	real*8		Temperature_Offset
	real*8		Salinity
	real*8		Depth_Offset
	logical*1	Header_Notes_Rec
	integer*4	Header_Notes_Len
	character*1	Header_Notes( String_Length )

!	LOCAL VARIABLES

	integer*4	L
	real*4		IBM_Software_Version
	integer*4	i
	integer*4	j
	real*8		Xducer_Angle

!	FUNCTION DECLARATIONS

	integer*4	Ick2
	integer*4	Bck1
	real*8		Rck8

		l=last_dat_flag
		ibm_software_version=float(ick2(hdr(l+1)))/100.
		l=l+14
		trans_up_down =  bck1(hdr(l+1))
		trans_cave_vex =  bck1(hdr(l+2))
		pings_per_ensemble = ick2(hdr(l+5))
		number_of_bins =  bck1(hdr(l+7))
		bin_length =  bck1(hdr(l+8))
	 	pulse_length =  bck1(hdr(l+9))
		blank_interval =  bck1(hdr(l+10))
		percent_good_threshold =  bck1(hdr(l+13))
		das_year =  bck1(hdr(l+14))
		das_month =  bck1(hdr(l+15))
		das_day =  bck1(hdr(l+16))
		das_hour =  bck1(hdr(l+17))
		das_minute =  bck1(hdr(l+18))
		das_second =  bck1(hdr(l+19))
		das_hund_of_sec =  bck1(hdr(l+20))
		id_length =  bck1(hdr(l+21))
		do i=l+22,l+21+id_length
			j=i-l-21
			cruise_id(j) = char(hdr(i))
		end do
 	call Initialize_Header_Flags( Number_Of_Header_Bytes,
     :				      l, id_length,
     :				      velocity_reference,
     :				      hdr, interface_type,
     :				      bottom_tracking_on,
     :				      transformed_data_on,
     :				      velx_data_rec,
     :				      vely_data_rec,
     :				      velz_data_rec,
     :				      velerr_data_rec,
     :				      roll_compensate_on,
     :				      pitch_compensate_on,
     :				      heading_compensate_on )
		l=l+21
		roll_offset = rck8(hdr(l+1))
		pitch_offset = rck8(hdr(l+9))
		heading_offset = rck8(hdr(l+17))
		sampling_interval = rck8(hdr(l+41))
		first_bin_ref_layer =  bck1(hdr(l+49))
		last_bin_ref_layer =  bck1(hdr(l+50))
		temperature_scale = rck8(hdr(l+59))
		temperature_offset = rck8(hdr(l+67))
		salinity = rck8(hdr(l+75))
		depth_offset = rck8(hdr(l+83))
		xducer_angle = rck8(hdr(l+91))
		l=l+98
		if(header_notes_rec)then
			header_notes_len = ick2(hdr(l+1))
			do i=l+3,l+2+header_notes_len
				j=i-l-2
				header_notes(j) = char(hdr(i))
			end do
			l=l+2+header_notes_len
		end if
	end !Assign_Header_Variables

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Initialize_Header_Flags( Number_Of_Header_Bytes,
     :					    l, id_length,
     :					    velocity_reference,
     :					    hdr, interface_type,
     :					    bottom_tracking_on,
     :					    transformed_data_on,
     :					    velx_data_rec,
     :					    vely_data_rec,
     :					    velz_data_rec,
     :					    velerr_data_rec,
     :					    roll_compensate_on,
     :					    pitch_compensate_on,
     :					    heading_compensate_on )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Header_Bytes
 	integer*4	l
	integer*2	id_length
	integer*2	velocity_reference
	byte		hdr( Number_Of_Header_Bytes )
	integer*2	interface_type
	logical*1	bottom_tracking_on
	logical*1	transformed_data_on
	logical*1	velx_data_rec
	logical*1	vely_data_rec
	logical*1	velz_data_rec
	logical*1	velerr_data_rec
	logical*1	roll_compensate_on
	logical*1	pitch_compensate_on
	logical*1	heading_compensate_on

!	FUNCTION DECLARATIONS

	integer*4	Bck1

		l = l+21+id_length
	        velocity_reference = bck1(hdr(l+1))
		interface_type =  bck1(hdr(l+2))
		if( bck1(hdr(l+3)).eq.255)bottom_tracking_on = .true.
		if( bck1(hdr(l+6)).eq.255)transformed_data_on = .true.
		if( bck1(hdr(l+7)).eq.255)velx_data_rec = .true.
		if( bck1(hdr(l+8)).eq.255)vely_data_rec = .true.
		if( bck1(hdr(l+9)).eq.255)velz_data_rec = .true.
		if( bck1(hdr(l+10)).eq.255)velerr_data_rec = .true.
		if( bck1(hdr(l+11)).eq.255)roll_compensate_on = .true.
		if( bck1(hdr(l+12)).eq.255)pitch_compensate_on = .true.
		if( bck1(hdr(l+13)).eq.255)heading_compensate_on = .true.
	end !Initialize_Header_Flags

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Compute_Data_Variables( Number_Of_Data_Bytes,
     :					   Ensemble_Year, Ensemble_Month,
     :					   Ensemble_Day, Ensemble_Hour,
     :					   Ensemble_Minute, Ensemble_Second,
     :					   Ensemble_Hund_Of_Sec,
     :					   Ensemble_Number, Data,
     :					   Samples_Per_Average, Hrpt_Stats_Rec,
     :					   Avg_Roll, Avg_Pitch, Avg_Heading,
     :					   Avg_Temp, Temperature_Offset,
     :					   Temperature_Scale, Std_Dev_Temp,
     :					   Std_Dev_Heading, Std_Dev_Pitch,
     :					   Std_Dev_Roll, L, ASCII_Unit_No,
     :					   ASCII_Output )
!	Subroutine Compute_Data_Variables computes statistics for ensemble
!	temperature, heading, pitch, and roll. 

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Data_Bytes
	integer*4	Ensemble_Year
	integer*4	Ensemble_Month
	integer*4	Ensemble_Day
	integer*4	Ensemble_Hour
	integer*4	Ensemble_Minute
	integer*4	Ensemble_Second
	integer*4	Ensemble_Hund_Of_Sec
	integer*2	Ensemble_Number
	byte		Data( Number_Of_Data_Bytes )
	integer*2	Samples_Per_Average
	logical*1	Hrpt_Stats_Rec
	real*8		Avg_Roll
	real*8		Avg_Pitch
	real*8		Avg_Heading
	real*8		Avg_Temp
	real*8		Temperature_Offset
	real*8		Temperature_Scale
	real*4		Std_Dev_Temp
	real*4		Std_Dev_Heading
	real*4		Std_Dev_Pitch
	real*4		Std_Dev_Roll
	integer*4	L
	integer*4	ASCII_Unit_No
	logical*1	ASCII_Output

!	LOCAL VARIABLES

	real*8		Most_Recent_Roll
	real*8		Most_Recent_Pitch
	real*8		Most_Recent_Heading
	real*8		Most_Recent_Temp
	real*8		Sum_Roll_Squared
	real*8		Sum_Pitch_Squared
	real*8		Sum_Heading_Squared
	real*8		Sum_Temp_Squared
	integer*2	Num_Samples_For_Avg
	real*4		Sum_Temp
	real*4		Arg
	real*4		Sum_Heading
	real*4		Sum_Pitch
	real*4		Sum_Roll

!	FUNCTION DECLARATIONS

	integer*4	ick2
	real*8		rck8

	call Assign_Ensemble_Data( Number_Of_Data_Bytes, Ensemble_Year,
     :				   Ensemble_Month, Ensemble_Day,
     :				   Ensemble_Hour, Ensemble_Minute,
     :				   Ensemble_Second, Ensemble_Hund_Of_Sec,
     :				   Ensemble_Number, Data )

		samples_per_average = ick2(data(14))
		l=17
		if(hrpt_stats_rec)then
		most_recent_roll = rck8(data(l+1))
		most_recent_pitch = rck8(data(l+9))
		most_recent_heading = rck8(data(l+17))
		most_recent_temp = rck8(data(l+25))
		avg_roll = rck8(data(l+33))
		avg_pitch = rck8(data(l+41))
		avg_heading = rck8(data(l+49))
		avg_temp = rck8(data(l+57))
		sum_roll_squared = rck8(data(l+65))
		sum_pitch_squared = rck8(data(l+73))
		sum_heading_squared = rck8(data(l+81))
		sum_temp_squared = rck8(data(l+89))
		num_samples_for_avg = ick2(data(l+97))
		most_recent_temp = temperature_offset - temperature_scale *
     :              most_recent_temp / 4096.
		sum_temp = avg_temp*num_samples_for_avg
		avg_temp = temperature_offset - temperature_scale *
     :              avg_temp / 4096.
		if(sum_temp_squared.gt..001)then
		arg = (sum_temp_squared - sum_temp*sum_temp/
     :             num_samples_for_avg) / num_samples_for_avg
		else
		arg=0.
		end if
		if(arg.ge.0)then
			std_dev_temp = sqrt ( arg )
		else
			std_dev_temp = 0.
		end if
		std_dev_temp = temperature_scale * std_dev_temp / 4096.
c
		sum_heading = avg_heading*num_samples_for_avg
		if(sum_heading_squared.gt..001)then
		if ( ( (sum_heading_squared - sum_heading*
     :               sum_heading/ num_samples_for_avg) / num_samples_for_avg )
     :		.lt. 0.0 ) then
		   std_dev_heading = 0.0
		else
 		   std_dev_heading = sqrt ( (sum_heading_squared - sum_heading*
     :               sum_heading/ num_samples_for_avg) / num_samples_for_avg)
		end if
		end if
		avg_heading = avg_heading*360./65536.
		std_dev_heading = std_dev_heading*360./65536.
		if(sum_pitch_squared.gt..001)then
		sum_pitch = avg_pitch*num_samples_for_avg
 		std_dev_pitch = sqrt ( (sum_pitch_squared - sum_pitch*
     :               sum_pitch/ num_samples_for_avg) / num_samples_for_avg)
		avg_pitch = avg_pitch*360./65536.
		std_dev_pitch = std_dev_pitch*360./65536.
		end if
		sum_roll = avg_roll*num_samples_for_avg
		if(sum_roll_squared.gt..001)then
 		std_dev_roll = sqrt ( (sum_roll_squared - sum_roll*
     :               sum_roll/ num_samples_for_avg) / num_samples_for_avg)
		avg_roll = avg_roll*360./65536.
		std_dev_roll = std_dev_roll*360./65536.
		end if
		l=l+99
		end if
	end !Compute_Data_Variables

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_To_Data_Array( Number_Of_Bytes_In_Record,
     :					 Number_of_Data_Bytes, Last_Byte,
     :					 Data, Line, Num_Data_Rec, 
     :					 New_Data_Rec )
!	Subroutine Assign_To_Data_Array assigns the inputted information into
!	an array called Data.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*2	Number_of_Data_Bytes
	integer*4	Last_Byte
	byte		Data( Number_of_Data_Bytes )
	byte		Line( Number_Of_Bytes_In_Record )
	integer*4	Num_Data_Rec
	integer*4	New_Data_Rec

!	LOCAL VARIABLES

	integer*2	Index

	do Index = last_byte + 1, last_byte + ( Number_Of_Bytes_In_Record - 2 )
           data( Index ) = line( Index - last_byte + 2 )
	end do
	last_byte = last_byte + ( Number_Of_Bytes_In_Record - 2 )
	num_data_rec = new_data_rec
	end !Assign_To_Data_Array

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_Ensemble_Data( Number_Of_Data_Bytes, Ensemble_Year,
     :					 Ensemble_Month, Ensemble_Day,
     :					 Ensemble_Hour, Ensemble_Minute,
     :					 Ensemble_Second, Ensemble_Hund_Of_Sec,
     :					 Ensemble_Number, Data )
!	Subroutine Assign_Ensemble_Data assigns ensemble time and ensemble
!	number information to variables.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Data_Bytes
	integer*4	Ensemble_Year
	integer*4	Ensemble_Month
	integer*4	Ensemble_Day
	integer*4	Ensemble_Hour
	integer*4	Ensemble_Minute
	integer*4	Ensemble_Second
	integer*4	Ensemble_Hund_Of_Sec
	integer*2	Ensemble_Number
	byte		Data( Number_Of_Data_Bytes )

!	FUNCTION DECLARATIONS

	integer*4	Bck1
	integer*4	Ick2

!	LOCAL VARIABLES

	integer*4	i4_Ensemble_Number

	ensemble_year =  bck1(data(5))
	ensemble_month =  bck1(data(6))
	ensemble_day =  bck1(data(7))
	ensemble_hour =  bck1(data(8))
	ensemble_minute =  bck1(data(9))
	ensemble_second =  bck1(data(10))
	ensemble_hund_of_sec =  bck1(data(11))
!	ensemble_number = ick2(data(12))
	i4_Ensemble_Number = ick2(data(12))
	if ( i4_Ensemble_Number .LT. '7FFF'X ) then
	   ensemble_number = i4_Ensemble_Number
	else
	   ensemble_number = i4_Ensemble_Number - 'FFFF'X
	end if
	end !Assign_Ensemble_Data

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Conditional_Assignments( Number_Of_Bytes_In_Record,
     :					    Number_Of_Columns,
     :					    Number_Of_Header_Bytes,
     :					    Number_Of_Data_Bytes,
     :					    User_Exit_Buf_Length,
     :					    Loran_Rec, Velocity_Data_Rec,
     :					    Velx_Data_Rec, Vely_Data_Rec,
     :					    Velz_Data_Rec, Velerr_Data_Rec,
     :					    Number_Of_Bins, Data,
     :					    L, Vel_X, Vel_Y, Vel_Z,
     :					    Vel_Err, Velocity_Stats_Rec,
     :					    Amp_Data_Rec, Avg_Amplitude,
     :					    Percent_Good_Rec, Percent_Good,
     :					    Bot_Track_Data_Rec, Ew_Bottom,
     :					    Ns_Bottom, Bottom_Depth,
     :					    Bottom_Sampling_Interval,
     :					    Loran_Latitude, Loran_Longitude,
     :					    Loran_Velocity, Loran_Direction,
     :					    User_Exit_Buf_Rec, User_Buf_Len,
     :					    Hdr, User_Exit_Buf, Ctd_Data_Rec,
     :					    Avg_Spec_Width_Rec, Avg_Spec_Width,
     :					    Raw_Dop_Data_Rec, Raw_Dop_Val,
     :					    Raw_Amp_Data_Rec, Raw_Amp_Val,
     :					    Raw_Spec_Width_Rec, Raw_Spec_Width,
     :					    Num_Three_Beam_Rec, Num_Three_Beam,
     :					    Beam_Quality_Stats_Rec, Beam_1_Good,
     :					    Beam_2_Good, Beam_3_Good,
     :					    Beam_4_Good, Std_Dev_Vel_Rec,
     :					    Std_Dev_Vel )
!	Subroutine Conditional_Assignments assigns optional information to
!	variable names.  The procedure checks the Data_Flag(s), s = 0,...,24,
!	to see if the information was input.  If a particular flag is .true.,
!	then the information is assigned to the appropriate variables.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	byte		Number_Of_Columns
	integer*2	Number_Of_Header_Bytes
	integer*2	Number_Of_Data_Bytes
	integer*2	User_Exit_Buf_Length
	logical*1	Loran_Rec
	logical*1	Velocity_Data_Rec
	logical*1	Velx_Data_Rec
	logical*1	Vely_Data_Rec
	logical*1	Velz_Data_Rec
	logical*1	Velerr_Data_Rec
	integer*2	Number_Of_Bins
	byte		Data( Number_Of_Data_Bytes )
	integer*4	L
	integer*2	Vel_X( Number_Of_Bytes_In_Record )
	integer*2	Vel_Y( Number_Of_Bytes_In_Record )
	integer*2	Vel_Z( Number_Of_Bytes_In_Record )
	integer*2	Vel_Err( Number_Of_Bytes_In_Record )
	logical*1	Velocity_Stats_Rec
	logical*1	Amp_Data_Rec
	integer*2	Avg_Amplitude( Number_Of_Bytes_In_Record )
	logical*1	Percent_Good_Rec
	integer*2	Percent_Good( Number_Of_Bytes_In_Record )
	logical*1	Bot_Track_Data_Rec
	real*8		Ew_Bottom
	real*8		Ns_Bottom
	real*8		Bottom_Depth
	real*8		Bottom_Sampling_Interval
	real*8		Loran_Latitude
	real*8		Loran_Longitude
	real*8		Loran_Velocity
	real*8		Loran_Direction
	logical*1	User_Exit_Buf_Rec
	integer*2	User_Buf_Len
	byte		Hdr( Number_Of_Header_Bytes )
	character*1	User_Exit_Buf( User_Exit_Buf_Length )
	logical*1	Ctd_Data_Rec
	logical*1	Avg_Spec_Width_Rec
	integer*2	Avg_Spec_Width( Number_Of_Bytes_In_Record )
	logical*1	Raw_Dop_Data_Rec
	integer*2	Raw_Dop_Val( Number_Of_Bytes_In_Record,
     :					Number_Of_Columns )
	logical*1	Raw_Amp_Data_Rec
	integer*2	Raw_Amp_Val( Number_Of_Bytes_In_Record,
     :					Number_Of_Columns )
	logical*1	Raw_Spec_Width_Rec
	integer*2	Raw_Spec_Width( Number_Of_Bytes_In_Record,
     :					Number_Of_Columns )
	logical*1	Num_Three_Beam_Rec
	integer*2	Num_Three_Beam( Number_Of_Bytes_In_Record )
	logical*1	Beam_Quality_Stats_Rec
	integer*2	Beam_1_Good
	integer*2	Beam_2_Good
	integer*2	Beam_3_Good
	integer*2	Beam_4_Good
	logical*1	Std_Dev_Vel_Rec
	integer*2	Std_Dev_Vel( Number_Of_Bytes_In_Record,
     :				     Number_Of_Columns )

!	LOCAL VARIABLES

	integer*4	Increment
	integer*4	I
	integer*4	J

!	FUNCTION DECLARATIONS

	integer*4	Ick2_Sign
	integer*4	Bck1
	real*8		Rck8
	integer*4	Ick2

		if(velocity_data_rec)then
 			increment=0
			if(velx_data_rec)increment=increment+2
			if(vely_data_rec)increment=increment+2
			if(velz_data_rec)increment=increment+2
			if(velerr_data_rec)increment=increment+2
		    if(velx_data_rec)then
			do i=1,number_of_bins
                           vel_x(i)=ick2_sign(data(l+(i-1)*increment+1))
			end do
		    end if
		    if(vely_data_rec)then
			do i=1,number_of_bins
                           vel_y(i)=ick2_sign(data(l+(i-1)*increment+3))
			end do
		    end if
		    if(velz_data_rec)then
			do i=1,number_of_bins
                           vel_z(i)=ick2_sign(data(l+(i-1)*increment+5))
			end do
		    end if
		    if(velerr_data_rec)then
			do i=1,number_of_bins
                           vel_err(i)=ick2_sign(data(l+(i-1)*increment+7))
			end do
		    end if
		l=l+increment*number_of_bins
   	   end if
	if(amp_data_rec)then
		do i=1,number_of_bins
			avg_amplitude(i)= bck1(data(l+i))
		end do
		l=l+number_of_bins
	end if
	if(percent_good_rec)then
		do i=1,number_of_bins
			percent_good(i)= bck1(data(l+i))
		end do
		l=l+number_of_bins
	end if
	if(bot_track_data_rec)then
		ew_bottom = rck8(data(l+1))
		ns_bottom = rck8(data(l+9))
		bottom_depth = rck8(data(l+17))
		bottom_sampling_interval = rck8(data(l+25))
		loran_latitude = rck8(data(l+33))
		loran_longitude = rck8(data(l+41))
		loran_velocity = rck8(data(l+49))
		loran_direction = rck8(data(l+57))
		l=l+80
	end if
		if(user_exit_buf_rec)then
			user_buf_len=ick2(data(l+1))			
			do i=l+3,l+2+user_buf_len
				j=i-l-2
				user_exit_buf(j) = char(data(i))
			end do
			l=l+2+user_buf_len
		end if
		if(ctd_data_rec)then
c			doubt we will use this
			l=l+66
		end if
		if(avg_spec_width_rec)then
			do i=1,number_of_bins
			  avg_spec_width(i)= bck1(data(l+i))
			end do
			l=l+number_of_bins
		end if
		if(raw_dop_data_rec)then
			do i=1,number_of_bins
			  raw_dop_val(i,1)=ick2( data( (l+(i-1)*8)+1 ) )
			  raw_dop_val(i,2)=ick2( data( (l+(i-1)*8)+3 ) )
			  raw_dop_val(i,3)=ick2( data( (l+(i-1)*8)+5 ) )
			  raw_dop_val(i,4)=ick2( data( (l+(i-1)*8)+7 ) )
			end do
			l=l+number_of_bins*8
		end if	
		if(raw_amp_data_rec)then
			do i=1,number_of_bins
			  raw_amp_val(i,1)= bck1( data( (l+(i-1)*4)+1 ) )
			  raw_amp_val(i,2)= bck1( data( (l+(i-1)*4)+2 ) )
			  raw_amp_val(i,3)= bck1( data( (l+(i-1)*4)+3 ) )
			  raw_amp_val(i,4)= bck1( data( (l+(i-1)*4)+4 ) )
			end do
			l=l+number_of_bins*4
		end if	
		if(raw_spec_width_rec)then
			do i=1,number_of_bins
			  raw_spec_width(i,1)= bck1( data( (l+(i-1)*4)+1 ) )
			  raw_spec_width(i,2)= bck1( data( (l+(i-1)*4)+2 ) )
			  raw_spec_width(i,3)= bck1( data( (l+(i-1)*4)+3 ) )
			  raw_spec_width(i,4)= bck1( data( (l+(i-1)*4)+4 ) )
			end do
			l=l+number_of_bins*4
		end if	
		if(num_three_beam_rec)then
			do i=1,number_of_bins
			  num_three_beam(i)= bck1(data(l+i))
			end do
			l=l+number_of_bins
		end if	
		if(beam_quality_stats_rec)then
			beam_1_good =  bck1(data(l+1))
			beam_2_good =  bck1(data(l+2))
			beam_3_good =  bck1(data(l+3))
			beam_4_good =  bck1(data(l+4))
			l=l+4
		end if
		if(std_dev_vel_rec)then
		l=l+1
			do i=1,number_of_bins
			  std_dev_vel(i,1)=ick2( data( (l+(i-1)*8)+1 ) )
			  std_dev_vel(i,2)=ick2( data( (l+(i-1)*8)+3 ) )
			  std_dev_vel(i,3)=ick2( data( (l+(i-1)*8)+5 ) )
			  std_dev_vel(i,4)=ick2( data( (l+(i-1)*8)+7 ) )
			end do
			l=l+number_of_bins*8
		end if	
	end !Conditional_Assignments

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Get_Julian_Time( Ensemble_Year, Ensemble_Month,
     :				    Ensemble_Day, Ensemble_Hour,
     :				    Ensemble_Minute, Ensemble_Second,
     :				    id, rd )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*4	Ensemble_Year
	integer*4	Ensemble_Month
	integer*4	Ensemble_Day
	integer*4	Ensemble_Hour
	integer*4	Ensemble_Minute
	integer*4	Ensemble_Second
	integer*2	id
	real*4		rd

!	FUNCTION DECLARATIONS

	real*4		grejul

c     	type *,ensemble_year,ensemble_month,ensemble_day
	if(ensemble_month .lt. 13 .and. ensemble_day .lt. 32 .and.
     :     ensemble_month.gt.0 .and. ensemble_day.gt.0)
     :  id = int(grejul(ensemble_year,ensemble_month,ensemble_day))
	rd = float(ensemble_hour)/24. + float(ensemble_minute)/1440. +
     :       float(ensemble_second)/86400.
	end !Get_Julian_Time

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_To_Data_Flags( data_flag,
     :					 bottom_tracking_on,
     :					 velx_data_rec,
     :					 vely_data_rec,
     :					 velz_data_rec,
     :					 velerr_data_rec,
     :					 roll_compensate_on,
     :					 pitch_compensate_on,
     :					 heading_compensate_on, 
     :					 hrpt_stats_rec,
     :					 ship_vel_stats_rec,
     :					 loran_rec,
     :					 velocity_data_rec,
     :					 velocity_stats_rec,
     :					 amp_data_rec,
     :					 percent_good_rec,
     :					 bot_track_data_rec,
     :					 user_exit_buf_rec,
     :					 ctd_data_rec,
     :					 avg_spec_width_rec,
     :					 raw_dop_data_rec,
     :					 raw_amp_data_rec,
     :					 raw_spec_width_rec,
     :					 num_three_beam_rec,
     :					 beam_quality_stats_rec,
     :					 std_dev_vel_rec )
!	Subroutine Assign_To_Data_Flags assigns the flags to an array called
!	Data_Flag.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	logical*1	data_flag( 0:24 )
	logical*1	bottom_tracking_on 
	logical*1	velx_data_rec 
	logical*1	vely_data_rec 
	logical*1	velz_data_rec 
	logical*1	velerr_data_rec 
	logical*1	roll_compensate_on
	logical*1	pitch_compensate_on
	logical*1	heading_compensate_on 
	logical*1	hrpt_stats_rec
	logical*1	ship_vel_stats_rec
	logical*1	loran_rec
	logical*1	velocity_data_rec
	logical*1	velocity_stats_rec
	logical*1	amp_data_rec
	logical*1	percent_good_rec
	logical*1	bot_track_data_rec
	logical*1	user_exit_buf_rec
	logical*1	ctd_data_rec
	logical*1	avg_spec_width_rec
	logical*1	raw_dop_data_rec
	logical*1	raw_amp_data_rec
	logical*1	raw_spec_width_rec
	logical*1	num_three_beam_rec
	logical*1	beam_quality_stats_rec
	logical*1	std_dev_vel_rec

	data_flag(0)=bottom_tracking_on 
	data_flag(1)=velx_data_rec 
	data_flag(2)=vely_data_rec 
	data_flag(3)=velz_data_rec 
	data_flag(4)=velerr_data_rec 
	data_flag(5)=roll_compensate_on
	data_flag(6)=pitch_compensate_on
	data_flag(7)=heading_compensate_on 
	data_flag(8)=hrpt_stats_rec
	data_flag(9)=ship_vel_stats_rec
	data_flag(10)=loran_rec
	data_flag(11)=velocity_data_rec
	data_flag(12)=velocity_stats_rec
	data_flag(13)=amp_data_rec
	data_flag(14)=percent_good_rec
	data_flag(15)=bot_track_data_rec
	data_flag(16)=user_exit_buf_rec
	data_flag(17)=ctd_data_rec
	data_flag(18)=avg_spec_width_rec
	data_flag(19)=raw_dop_data_rec
	data_flag(20)=raw_amp_data_rec
	data_flag(21)=raw_spec_width_rec
	data_flag(22)=num_three_beam_rec
	data_flag(23)=beam_quality_stats_rec
	data_flag(24)=std_dev_vel_rec
	end !Assign_To_Data_Flags

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Make_Corrections_If_Requested( Num_Of_Corrections, id,
     :						  rd, ntc,
     :						  Implement_Sound_Correction,
     :						  Implement_Time_Correction,
     :						  Avg_Temp, Salinity, Rv,
     :						  Sound_Vel_Factor,
     :						  Time_Corrections )
!	Subroutine Make_Corrections_If_Requested calls a routine to compute
!	the correction factor for the sound velocity and/or to correct the IBM
!	time recordings, as requested.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Num_Of_Corrections
	integer*2	id
	real*4		rd
	integer*2	ntc
	logical*1	Implement_Sound_Correction
	logical*1	Implement_Time_Correction
	real*8		Avg_Temp
	real*8		Salinity
	real*4		Rv
	real*4		Sound_Vel_Factor
	structure	/ Time_Corrections_Record /
	   real*4	Start_Time
	   real*4	Stop_Time
	   real*8	Intercept
	   real*8	Slope
	end structure
	record /Time_Corrections_Record/ Time_Corrections( Num_Of_Corrections )

	if ( Implement_Sound_Correction ) then
	   call rdi_sound(avg_temp,salinity,rv)
	   call Assign_To_Sound_Vel_Factor( Avg_Temp, Rv, Sound_Vel_Factor )
	else
	   rv		    = 1536.0
	   Sound_Vel_Factor =	 1
	end if
	if ( Implement_Time_Correction )
     :	   call Correct_For_IBM_Time( Num_Of_Corrections, Time_Corrections,
     :				      id, rd, ntc )
	end !Make_Corrections_If_Requested

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Correct_For_IBM_Time( Num_Of_Corrections, Time_Corrections,
     :					 id, rd, ntc )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Num_Of_Corrections
	structure	/ Time_Corrections_Record /
	   real*4	Start_Time
	   real*4	Stop_Time
	   real*8	Intercept
	   real*8	Slope
	end structure
	record /Time_Corrections_Record/ Time_Corrections( Num_Of_Corrections )
	integer*2	id
	real*4		rd
	integer*2	ntc

!	LOCAL VARIABLES

	real*8		IBM_Time
	real*8		New_IBM_Time
	integer*2	Index

	   ibm_time = float(id)+rd
	   do Index=1,ntc
	    if(ibm_time.gt.Time_Corrections(Index).Start_Time .and.
     :		ibm_time.lt.Time_Corrections(Index).Stop_Time)then
		new_ibm_time = Time_Corrections(Index).Intercept
     :			+ Time_Corrections(Index).Slope*ibm_time
		id = int (sngl (new_ibm_time) )
		rd = new_ibm_time - dble ( float ( id ) )
	    end if
	   end do
	end !Correct_For_IBM_Time

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_To_Sound_Vel_Factor( Avg_Temp, Rv, Sound_Vel_Factor )

	IMPLICIT NONE

!	FORMAL PARAMETERS

	real*8		Avg_Temp
	real*4		Rv
	real*4		Sound_Vel_Factor

!	FUNCTION DECLARATIONS

	logical*1	Avg_Temp_And_Sound_Vel!_Are_Within_Bounds

	if ( Avg_Temp_And_Sound_Vel( Avg_Temp, Rv ))!_Are_Within_Bounds
     :  then
		sound_vel_factor = rv / 1536.
	else
		rv = 1536.
		sound_vel_factor = 1.
	end if
	end !Assign_To_Sound_Vel_Factor


	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function Avg_Temp_And_Sound_Vel( Avg_Temp, Sound_Velocity )
!	or more appropriately, Avg_Temp_And_Sound_Vel_Are_Within_Bounds
!	Function Avg_Temp_And_Sound_Vel returns true if the average temperature
!	and average sound velocity are within preselected bounds, and false
!	otherwise.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	real*8		Avg_Temp
	real*4		Sound_Velocity
	logical*1	Avg_Temp_And_Sound_Vel!_Are_Within_Bounds

	Avg_Temp_And_Sound_Vel!_Are_Within_Bounds
     :		=  ( avg_temp .gt. 0. .and. avg_temp .lt. 40.
     :		     .and. Sound_Velocity .gt. 1400.
     :		     .and. Sound_Velocity .lt. 1600. )
	return 
	end !Avg_Temp_And_Sound_Vel!_Are_Within_Bounds

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Velocity_Assignments( Number_Of_Bytes_In_Record,
     :					 Data_Flag, Number_Of_Bins,
     :					 vx, vel_x, vy, vel_y,
     :					 vz, vel_z, ve, vel_err,
     :					 Sound_Vel_Factor )
!	Subroutine Velocity_Assignments computes the corrected velocity
!	assignments.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	logical*1	Data_Flag( 0:24 )
	integer*2	Number_Of_Bins
	real*4		vx( Number_Of_Bytes_In_Record )
	integer*2	vel_x( Number_Of_Bytes_In_Record )
	real*4		vy( Number_Of_Bytes_In_Record )
	integer*2	vel_y( Number_Of_Bytes_In_Record )
	real*4		vz( Number_Of_Bytes_In_Record )
	integer*2	vel_z( Number_Of_Bytes_In_Record )
	real*4		ve( Number_Of_Bytes_In_Record )
	integer*2	vel_err( Number_Of_Bytes_In_Record )
	real*4		Sound_Vel_Factor 

!	LOCAL VARIABLES

	integer*2	i

	if(data_flag(11) .and. data_flag(1))then
		do i=1,number_of_bins
			vx(i)=sound_vel_factor * vel_x(i)/10.
		end do
	end if
	if(data_flag(11) .and. data_flag(2))then
		do i=1,number_of_bins
			vy(i)=sound_vel_factor * vel_y(i)/10.
		end do
	end if
	if(data_flag(11) .and. data_flag(3))then
		do i=1,number_of_bins
			vz(i)=sound_vel_factor * vel_z(i)/10.
		end do
	end if
	if(data_flag(11) .and. data_flag(4))then
		do i=1,number_of_bins
			ve(i)=sound_vel_factor * vel_err(i)/10.
		end do
	end if
	end !Velocity_Assignments

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Establish_First_Ensemble_Info( Ensemble_Counter,
     :						  First_Ensemble_Processed,
     :						  Ensemble_Number,
     :						  First_Ensemble_Time,
     :						  id, rd,
     :						  First_Ensemble_Year,
     :						  Ensemble_Year )
!	Subroutine Establish_First_Ensemble_Info increments Ensemble_Counter
!	and saves the First_Ensemble_Number, the First_Ensemble_Time,
!	and the First_Ensemble_Year.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*4	Ensemble_Counter
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

	Ensemble_Counter = Ensemble_Counter + 1
	if ( First_Ensemble_Processed .EQ. 0 )
     :		First_Ensemble_Processed = Ensemble_Number
	if ( First_Ensemble_Time .EQ. 0 ) then
	   First_Ensemble_Time = id + rd
	   First_Ensemble_Year = Ensemble_Year
	end if
	end !Establish_First_Ensemble_Info

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_To_RealX4_Variables( Number_Of_Bytes_In_Record,
     :					       Number_Of_Columns, rw,
     :					       Bin_Length, rx,
     :					       Pulse_Length, rb,
     :					       Blank_Interval, rr,
     :					       Roll_Offset, rp,
     :					       Pitch_Offset, rh,
     :					       Heading_Offset, ro,
     :					       Depth_Offset, rs,
     :					       Sampling_Interval, rf,
     :					       First_Bin_Ref_Layer, rl,
     :					       Last_Bin_Ref_Layer, Data_Flag,
     :					       rat, Avg_Temp, rah,
     :					       Avg_Heading, rap,
     :					       Avg_Pitch, rar,
     :					       Avg_Roll, Number_Of_Bins,
     :					       Amp, Avg_Amplitude,
     :					       Pctgd, Percent_Good,
     :					       re, EW_Bottom, rn,
     :					       NS_Bottom, rz, Bottom_Depth,
     :					       avsw, Avg_Spec_Width,
     :					       rdpv, Raw_Dop_Val,
     :					       ramp, Raw_Amp_Val,
     :					       rspw, Raw_Spec_Width,
     :					       n3b, Num_Three_Beam,
     :					       bmg, Beam_1_Good,
     :					       Beam_2_Good, Beam_3_Good,
     :					       Beam_4_Good, stdv,
     :					       Std_Dev_Vel )
!	Subroutine Assign_To_RealX4_Variables assigns certain variables
!	to real*4 variable names.  The variable names are output to the
!	unformatted file if selected.  Some of the real*4 variable names
!	are output to the formatted file.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	byte		Number_Of_Columns
	real*4		rw
	integer*2	Bin_Length
	real*4		rx
	integer*2	Pulse_Length
	real*4		rb
	integer*2	Blank_Interval
	real*4		rr
	real*8		Roll_Offset
	real*4		rp
	real*8		Pitch_Offset
	real*4		rh
	real*8		Heading_Offset
	real*4		ro
	real*8		Depth_Offset
	real*4		rs
	real*8		Sampling_Interval
	real*4		rf
	integer*2	First_Bin_Ref_Layer
	real*4		rl
	integer*2	Last_Bin_Ref_Layer
	logical*1	Data_Flag( 0:24 )
	real*4		rat
	real*8		Avg_Temp
	real*4		rah
	real*8		Avg_Heading
	real*4		rap
	real*8		Avg_Pitch
	real*4		rar
	real*8		Avg_Roll
	integer*2	Number_Of_Bins
	real*4		Amp( Number_Of_Bytes_In_Record )
	integer*2	Avg_Amplitude( Number_Of_Bytes_In_Record )
	real*4		Pctgd( Number_Of_Bytes_In_Record )
	integer*2	Percent_Good( Number_Of_Bytes_In_Record )
	real*4		re
	real*8		EW_Bottom
	real*4		rn
	real*8		NS_Bottom
	real*4		rz
	real*8		Bottom_Depth
	real*4		avsw( Number_Of_Bytes_In_Record )
	integer*2	Avg_Spec_Width( Number_Of_Bytes_In_Record )
	real*4		rdpv( Number_Of_Bytes_In_Record, Number_Of_Columns )
	integer*2	Raw_Dop_Val( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		ramp( Number_Of_Bytes_In_Record, Number_Of_Columns )
	integer*2	Raw_Amp_Val( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		rspw( Number_Of_Bytes_In_Record, Number_Of_Columns )
	integer*2	Raw_Spec_Width( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		n3b( Number_Of_Bytes_In_Record )
	integer*2	Num_Three_Beam( Number_Of_Bytes_In_Record )
	real*4		bmg( Number_Of_Columns )
	integer*2	Beam_1_Good
	integer*2	Beam_2_Good
	integer*2	Beam_3_Good
	integer*2	Beam_4_Good
	real*4		stdv( Number_Of_Bytes_In_Record, Number_Of_Columns )
	integer*2	Std_Dev_Vel( Number_Of_Bytes_In_Record, Number_Of_Columns )

!	LOCAL VARIABLES

	integer*4	i
	integer*4	j

	rw = bin_length
	rx = pulse_length
	rb = blank_interval
	rr = roll_offset
	rp = pitch_offset
	rh = heading_offset
	rs = sampling_interval
	rf = first_bin_ref_layer
	rl = last_bin_ref_layer
	ro = depth_offset

	if(data_flag(8))then
		rat = avg_temp
		rah = avg_heading
		rap = avg_pitch
		rar = avg_roll
	end if

	if(data_flag(13))then
		do i=1,number_of_bins
			amp(i)=avg_amplitude(i)
		end do
	end if
	if(data_flag(14))then
		do i=1,number_of_bins
			pctgd(i)=percent_good(i)
		end do
	end if
	if(data_flag(15))then
		re = ew_bottom / 10.
		rn = ns_bottom / 10.
		rz = bottom_depth
	end if
	if(data_flag(18))then
		do i=1,number_of_bins
			avsw(i)=avg_spec_width(i)
		end do
	end if
	if(data_flag(19))then
		do i=1,number_of_bins
		   do j=1, NUMBER_OF_COLUMNS
			rdpv(i,j)=raw_dop_val(i,j)
		   end do
		end do
	end if
	if(data_flag(20))then
		do i=1,number_of_bins
		   do j=1, NUMBER_OF_COLUMNS
			ramp(i,j)=raw_amp_val(i,j)
		   end do
		end do
	end if
	if(data_flag(21))then
		do i=1,number_of_bins
		   do j=1, NUMBER_OF_COLUMNS
			rspw(i,j)=raw_spec_width(i,j)
		    end do
		end do
	end if
	if(data_flag(22))then
		do i=1,number_of_bins
			n3b(i)=num_three_beam(i)
		end do
	end if
	if(data_flag(23))then
		bmg(1) = beam_1_good
		bmg(2) = beam_2_good
		bmg(3) = beam_3_good
		bmg(4) = beam_4_good
 	end if
	if(data_flag(24))then
		do i=1,number_of_bins
		   do j=1, NUMBER_OF_COLUMNS
			stdv(i,j)=std_dev_vel(i,j)
		    end do
		end do
	end if
	end !Assign_To_RealX4_Variables

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Output_To_Binary_File( Number_Of_Bytes_In_Record,
     :					  Number_Of_Columns,
     :					  iy, id, Number_Of_Bins,
     :					  it, ie, ip,
     :					  rd, rw, rx,
     :					  rb, rr, rp,
     :				  	  rh, rs, rf,
     :					  rl, ro, rv,
     :					  rat, rsdt, rah,
     :					  rsdh, rap,
     :					  rsdp, rar,
     :					  rsdr, re,
     :					  rn, rz,
     :					  Data_Flag, vx,
     :					  vy, vz, ve,
     :					  amp, pctgd,
     :					  iunit, avsw,
     :					  rdpv, ramp,
     :					  rspw, n3b,
     :					  bmg, stdv )
!	Subroutine Output_To_Binary_File outputs information to the user-named
!	unformatted file, if selected.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	byte		Number_Of_Columns
	integer*2	iy
	integer*2	id
	integer*2	Number_Of_Bins
	integer*2	it
	integer*2	ie
	integer*2	ip
	real*4		dummy_vax_time         /0.0/
	real*4		dummy_gps_decimal_part /0.0/
	real*4		rd
	real*4		rw
	real*4		rx
	real*4		rb
	real*4		rr
	real*4		rp
	real*4		rh
	real*4		rs
	real*4		rf
	real*4		rl
	real*4		ro
	real*4		rv
	real*4		rat
	real*4		rsdt
	real*4		rah
	real*4		rsdh
	real*4		rap
	real*4		rsdp
	real*4		rar
	real*4		rsdr
	real*4		re
	real*4		rn
	real*4		rz
	logical*1	Data_Flag( 0:24 )
	real*4		vx( Number_Of_Bytes_In_Record )
	real*4		vy( Number_Of_Bytes_In_Record )
	real*4		vz( Number_Of_Bytes_In_Record )
	real*4		ve( Number_Of_Bytes_In_Record )
	real*4		amp( Number_Of_Bytes_In_Record )
	real*4		pctgd( Number_Of_Bytes_In_Record )
	integer*2	iunit
	real*4		avsw( Number_Of_Bytes_In_Record )
	real*4		rdpv( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		ramp( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		rspw( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		n3b( Number_Of_Bytes_In_Record )
	real*4		bmg( Number_Of_Columns )
	real*4		stdv( Number_Of_Bytes_In_Record, Number_Of_Columns )

!	LOCAL VARIABLES

	integer*2	i
	integer*2	j

	write(iunit)iy,dummy_vax_time,id,Number_Of_Bins,it,ie,
     :	ip,rd,rw,rx,rb,rr,rp,rh,rs,rf,rl,ro,
     :  rv,rat,rsdt,rah,rsdh,rap,rsdp,rar,rsdr,re,rn,rz,dummy_gps_decimal_part,
     :  (data_flag(i),i=0,24),(vx(i),vy(i),vz(i),ve(i),amp(i),pctgd(i),
     :  i=1,number_of_bins)

	if(data_flag(18))write(iunit)(avsw(i),i=1,number_of_bins)
	if(data_flag(19))write(iunit)((rdpv(i,j),i=1,number_of_bins),
     :						 j=1,Number_Of_Columns)
	if(data_flag(20))write(iunit)((ramp(i,j),i=1,number_of_bins),
     :						 j=1,Number_Of_Columns)
	if(data_flag(21))write(iunit)((rspw(i,j),i=1,number_of_bins),
     :						 j=1,Number_Of_Columns)
	if(data_flag(22))write(iunit)(n3b(i),i=1,number_of_bins)
	if(data_flag(23))write(iunit)(bmg(j),j=1,Number_Of_Columns)
     	if(data_flag(24))write(iunit)((stdv(i,j),i=1,number_of_bins),
     :						 j=1,Number_Of_Columns)
	end !Output_To_Binary_File

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Output_To_ASCII_File( Number_Of_Bytes_In_Record,
     :					 Number_Of_Columns,
     :				         Ping_File_Name, ASCII_Unit_No,
     :				         Brief_Output, Ensemble_Year, id,
     :				         Number_Of_Bins, Percent_Good_Threshold,
     :				         Ensemble_Number, ip,
     :				         rd, Bin_Length, Pulse_Length,
     :				         Blank_Interval, Roll_Offset,
     :					 Pitch_Offset, Heading_Offset, 
     :				         Sampling_Interval,
     :					 First_Bin_Ref_Layer,
     :				         Last_Bin_Ref_Layer,
     :				    	 Depth_Offset, Sound_Velocity,
     :				    	 Avg_Temp, Std_Dev_Temp, Avg_Heading,
     :				    	 Std_Dev_Heading, Avg_Pitch,
     :				    	 Std_Dev_Pitch, Avg_Roll,
     :				    	 Std_Dev_Roll, East_Vel_Of_Ship,
     :				    	 North_Vel_Of_Ship, Water_Depth,
     :				    	 Data_Flag, vx,
     :				    	 vy, vz, ve,
     :				    	 amp, pctgd,
     :				    	 avsw,
     :				    	 rdpv, ramp,
     :				    	 rspw, n3b,
     :				    	 bmg, stdv )
!	Subroutine Output_To_Binary_File outputs information to the user-named
!	formatted file, if selected.  Some information is optionally output
!	depending on whether the Brief_Output flag was raised or not.  If the
!	user selects a formatted file option, then she/he is given a
!	full output / brief output option.  The Brief_Output flag is raised
!	if the brief output option is selected.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	byte		Number_Of_Columns
	character*(*)	Ping_File_Name
	integer*4	ASCII_Unit_No
	logical*1	Brief_Output
	integer*4	Ensemble_Year
	integer*2	id
	integer*2	Number_Of_Bins
	integer*2	Percent_Good_Threshold
	integer*2	Ensemble_Number
	integer*2	ip
	real*4		rd
	integer*2	Bin_Length
	integer*2	Pulse_Length
	integer*2	Blank_Interval
	real*8		Roll_Offset
	real*8		Pitch_Offset
	real*8		Heading_Offset
	real*8		Sampling_Interval
	integer*2	First_Bin_Ref_Layer
	integer*2	Last_Bin_Ref_Layer
	real*8		Depth_Offset
	real*4		Sound_Velocity
	real*8		Avg_Temp
	real*4		Std_Dev_Temp
	real*8		Avg_Heading
	real*4		Std_Dev_Heading
	real*8		Avg_Pitch
	real*4		Std_Dev_Pitch
	real*8		Avg_Roll
	real*4		Std_Dev_Roll
	real*4		East_Vel_Of_Ship
	real*4		North_Vel_Of_Ship
	real*4		Water_Depth
	logical*1	Data_Flag( 0:24 )
	real*4		vx( Number_Of_Bytes_In_Record )
	real*4		vy( Number_Of_Bytes_In_Record )
	real*4		vz( Number_Of_Bytes_In_Record )
	real*4		ve( Number_Of_Bytes_In_Record )
	real*4		amp( Number_Of_Bytes_In_Record )
	real*4		pctgd( Number_Of_Bytes_In_Record )
	real*4		avsw( Number_Of_Bytes_In_Record )
	real*4		rdpv( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		ramp( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		rspw( Number_Of_Bytes_In_Record, Number_Of_Columns )
	real*4		n3b( Number_Of_Bytes_In_Record )
	real*4		bmg( Number_Of_Columns )
	real*4		stdv( Number_Of_Bytes_In_Record, Number_Of_Columns )

!	LOCAL VARIABLES

	integer*2	Index
	integer*2	Row_Index
	integer*2	Column_Index
	real*4		Julian_Date
	integer*2	Bin_Length_In_Meters
	integer*2	i
	integer*2	j

	write( ASCII_Unit_No, 200 )
     :		(' Output from raw RDI ADCP file:  ', Ping_File_Name )
200	format( ///2a )
	write( ASCII_Unit_No, 300 ) ( ' Ensemble  ', Ensemble_Number )
300	format( //,a, i4 )
	Julian_Date = id + rd
	write( ASCII_Unit_No, 305 ) ( ' Julian Date, Year:  ', Julian_Date,
     :				       Ensemble_Year )
305	format( a, f9.5, i4 )
	write( ASCII_Unit_No, 310 ) ( ip, ' pings per ensemble' )
310	format( i3, a )
	Bin_Length_In_Meters = 2**Bin_Length
	if ( Number_Of_Bins .LT. 100 ) write( ASCII_Unit_No, 320 )
     :					    ( Number_Of_Bins, ' depth bins of ',
     :					      Bin_Length_In_Meters,
     :					      ' meters length' )
320	format( 2( i3, a ) )
	if ( Number_Of_Bins .GE. 100 ) write( ASCII_Unit_No, 325 )
     :					    ( Number_Of_Bins, ' depth bins of ',
     :					      Bin_Length_In_Meters,
     :					      ' meters length' )
325	format( i4, a, i3, a  )
	write( ASCII_Unit_No, 330 ) ( ' Percent Good Threshold:  ',
     :					Percent_Good_Threshold )
330	format( a, i3 )
	write( ASCII_Unit_No, 340 ) ( ' Pulse Length:  ', Pulse_Length )
340	format( a, i4 )
	write( ASCII_Unit_No, 350 ) ( ' Blank Interval:  ', Blank_Interval )
350	format( a, i4 )
	write( ASCII_Unit_No, 360 ) ( ' Roll Offset:  ', Roll_Offset )
	write( ASCII_Unit_No, 360 ) ( ' Pitch Offset:  ', Pitch_Offset )
	write( ASCII_Unit_No, 360 ) ( ' Heading Offset:  ', Heading_Offset )
360	format( a, f9.5 )
	write( ASCII_Unit_No, 370 ) ( ' Sampling Interval:  ',
     :					INT( Sampling_Interval ) )
370	format( a, i4 )
	write( ASCII_Unit_No, 380 ) ( ' First Bin Reference Layer:  ',
     :					First_Bin_Ref_Layer )
	write( ASCII_Unit_No, 380 ) ( ' Last Bin Reference Layer:  ',
     :					Last_Bin_Ref_Layer )
380	format( a, i4 )
	write( ASCII_Unit_No, 390 ) ( ' Depth Offset:  ', Depth_Offset )
390	format( a, f9.5 )
	write( ASCII_Unit_No, 400 ) ( ' Sound Velocity:  ', Sound_Velocity )
400	format( a, f9.1 )
	write( ASCII_Unit_No, 410 ) ( ' Average Temperature:  ', Avg_Temp )
410	format( a, f9.2 )
	write( ASCII_Unit_No, 420 ) ( ' Temperature Standard Deviation:  ',
     :					Std_Dev_Temp )
420	format( a, f9.5 )
	write( ASCII_Unit_No, 430 ) ( ' Average Heading:  ', Avg_Heading )
430	format( a, f9.1 )
	write( ASCII_Unit_No, 440 ) ( ' Heading_Standard Deviation:  ',
     :					Std_Dev_Heading )
440	format( a, f9.3 )
	write( ASCII_Unit_No, 450 ) ( ' Average Pitch:  ', Avg_Pitch )
450	format( a, f9.3 )
	write( ASCII_Unit_No, 460 ) ( ' Pitch Standard Deviation:  ',
     :					Std_Dev_Pitch )
460	format( a, f9.3 )
	write( ASCII_Unit_No, 470 ) ( ' Average Roll:  ', Avg_Roll )
470	format( a, f9.3 )
	write( ASCII_Unit_No, 480 ) ( ' Roll Standard Deviation:  ',
     :					Std_Dev_Roll )
480	format( a, f9.3 )
	write( ASCII_Unit_No, 490 ) ( ' East Velocity of Ship:  ',
     :					East_Vel_Of_Ship )
490	format( a, f9.1 )
	write( ASCII_Unit_No, 500 ) ( ' North Velocity of Ship:  ',
     :					North_Vel_Of_Ship )
500	format( a, f9.1 )
	write( ASCII_Unit_No, 510 ) ( ' Water Depth:  ', Water_Depth )
510	format( a, f9.1 )

!Data Recording Flags

	write( ASCII_Unit_No, 559 ) ( ' CURRENT DATA RECORDING FLAG SETTINGS')
559	format( ///,a,/ )
	write( ASCII_Unit_No, 600 ) (' Bottom Tracking On = ',
     :					data_flag( 0 ) )
	write( ASCII_Unit_No, 600 ) (' Velx Data Rec = ',
     :					data_flag( 1 ) )
	write( ASCII_Unit_No, 600 ) (' Vely Data Rec = ',
     :					data_flag( 2 ) )
	write( ASCII_Unit_No, 600 ) (' Velz Data Rec = ',
     :					data_flag( 3 ) )
	write( ASCII_Unit_No, 600 ) (' Velerr Data Rec = ',
     :					data_flag( 4 ) )
	write( ASCII_Unit_No, 600 ) (' Roll Compensate On = ',
     :					data_flag( 5 ) )
	write( ASCII_Unit_No, 600 ) (' Pitch Compensate On = ',
     :					data_flag( 6 ) )
	write( ASCII_Unit_No, 600 ) (' Heading Compensate On = ',
     :					data_flag( 7 ) )
	write( ASCII_Unit_No, 600 ) (' Heading/Roll/Pitch/Temp Stats Rec = ',
     :					data_flag( 8 ) )
	write( ASCII_Unit_No, 600 ) (' Ship Velocity Stats Rec = ',
     :					data_flag( 9 ) )
	write( ASCII_Unit_No, 600 ) (' Loran Rec = ',
     :					data_flag( 10 ) )
	write( ASCII_Unit_No, 600 ) (' Velocity Data Rec = ',
     :					data_flag( 11 ) )
	write( ASCII_Unit_No, 600 ) (' Velocity Stats Rec = ',
     :					data_flag( 12 ) )
	write( ASCII_Unit_No, 600 ) (' Amp Data Rec = ',
     :					data_flag( 13 ) )
	write( ASCII_Unit_No, 600 ) (' Percent Good Rec = ',
     :					data_flag( 14 ) )
	write( ASCII_Unit_No, 600 ) (' Bottom Track Data Rec = ',
     :					data_flag( 15 ) )
	write( ASCII_Unit_No, 600 ) (' User Exit Buffer Rec = ',
     :					data_flag( 16 ) )
	write( ASCII_Unit_No, 600 ) (' CTD Data Rec = ',
     :					data_flag( 17 ) )
	write( ASCII_Unit_No, 600 ) (' Averaged Spectral Width Rec = ',
     :					data_flag( 18 ) )
	write( ASCII_Unit_No, 600 ) (' Raw Doppler Data Rec = ',
     :					data_flag( 19 ) )
	write( ASCII_Unit_No, 600 ) (' Raw Amplitude Data Rec = ',
     :					data_flag( 20 ) )
	write( ASCII_Unit_No, 600 ) (' Raw Spectral Width Rec = ',
     :					data_flag( 21 ) )
	write( ASCII_Unit_No, 600 ) (' Number of 3-beam Solutions Rec = ',
     :					data_flag( 22 ) )
	write( ASCII_Unit_No, 600 ) (' Beam Quality Statistics Rec = ',
     :					data_flag( 23 ) )
	write( ASCII_Unit_No, 600 ) (' Velocity Standard Deviations Rec = ',
     :					data_flag( 24 ) )
600	format( a, l2 )

!Standard Velocity data

	if ( data_flag( 11 ) ) then
	   write( ASCII_Unit_No, 695 )('    BIN      VX        VY        VZ',
     :				       '        VE       AMP     PCT GOOD' )
695	   format( ///2a/ )
	   do Index = 1, 3
	      write( ASCII_Unit_No, 700 ) ( Index, Vx( Index ), Vy( Index ),
     :					    Vz( Index ), Ve( Index ),
     :					    Amp( Index ), Pctgd( Index ) )
	   end do
	end if
	if ( Brief_Output .EQ. .false. ) then
	   if ( data_flag( 11 ) ) then
	      do Index = 4, Number_Of_Bins
	         write( ASCII_Unit_No, 700 ) ( Index, Vx( Index ), Vy( Index ),
     :					       Vz( Index ), Ve( Index ),
     :					       Amp( Index ), Pctgd( Index ) )
	      end do
700	      format( i6, 6( 3x, f7.1 ) )
	   end if

!_______CUT OFF HERE FOR BRIEF OUTPUT_________________________________
!(FOR BRIEF WRITE ONLY BINS 1,2,3 OF STANDARD VELOCITY OUTPUT)

  	   if( data_flag( 23 ) ) then
	      write( ASCII_Unit_No, 705 ) ( ' BEAM 1 GOOD        BEAM 2 GOOD',
     :					  '        BEAM 3 GOOD',
     :					  '        BEAM 4 GOOD' )
705	format( ///,3a, / )
	      write( ASCII_Unit_No, 710 ) ( NINT ( bmg( j ) ), j = 1,
     :							    Number_Of_Columns )
	   end if
710	format( i9, 3i19 )
  	   if ( Data_Flag( 18 ) ) then
	      write( ASCII_Unit_No, 715 ) ( '   BIN         AVG SPEC WIDTH' )
715	format( ///a/ )
	      do Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No, 720 ) ( Index, avsw( Index ) )
	      end do
720	format( i5,  14x, f5.1 )
	   end if
  	   if ( Data_Flag( 22 ) ) then
	      write( ASCII_Unit_No, 722 )('BIN            # 3-BEAM SOLUTION' )
722	format( ///3x,a/ )
	      do Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No, 723 ) ( Index, NINT( n3b( Index ) ) )
	      end do
723	format( i5,  19x, i5 )
	   end if
	   if ( Data_Flag( 24 ) ) then
	      write( ASCII_Unit_No, 725 )
     :			( ' STANDARD DEVIATION BY BEAM AND BIN #' )
	      write( ASCII_Unit_No, 730 )
	      do Row_Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No,  740 ) ( Row_Index,
     :					      ( Stdv( Row_Index, Column_Index ),
     :					      Column_Index = 1,
     :					      		  Number_Of_Columns ) )
	      end do
	   end if
	   if ( Data_Flag( 19 ) ) then
	      write( ASCII_Unit_No, 725 )
     :			( ' RAW DOPPLER VALUES BY BEAM AND BIN #' )
	      write( ASCII_Unit_No, 730 )
	      do Row_Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No, 740 )( Row_Index,
     :					      ( Rdpv( Row_Index, Column_Index ),
     :					      Column_Index = 1,
     :							  Number_Of_Columns ) )
	      end do
	   end if
	   if ( Data_Flag( 20 ) ) then
	      write( ASCII_Unit_No, 725 )
     :			( ' RAW AMPLITUDE VALUES BY BEAM AND BIN #' )
	      write( ASCII_Unit_No, 730 )
	      do Row_Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No, 740 )( Row_Index,
     :					      ( Ramp( Row_Index, Column_Index ),
     :					      Column_Index = 1,
     :							  Number_Of_Columns ) )
	      end do
	   end if
	   if ( Data_Flag( 21 ) ) then
	      write( ASCII_Unit_No, 725 )
     :			( ' RAW SPECTRAL WIDTH VALUES BY BEAM AND BIN #' )
	      write( ASCII_Unit_No, 730 )
	      do Row_Index = 1, Number_Of_Bins
	         write( ASCII_Unit_No, 740 ) ( Row_Index,
     :					      ( Rspw( Row_Index, Column_Index ),
     :					      Column_Index = 1,
     :							  Number_Of_Columns ) )
	      end do
	   end if
725	format( ///a )
730	format( //,'   BIN       BEAM 1       BEAM 2         BEAM 3',
     :		'        BEAM 4', / )
740	format( i5, 4(4x, f10.3) )
	end if !Brief Output
	end !Output_To_ASCII_File

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_To_New_Block_Type( New_Block_Type, Line_3_Byte )
!	Subroutine Assign_To_New_Block_Type assigns an integer expression to
!	variable New_Block_Type.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*4	New_Block_Type
	byte		Line_3_Byte

!	FUNCTION DECLARATIONS

	integer*4	ick2

	New_Block_Type = ick2( Line_3_Byte )
	end !Assign_To_New_Block_Type


	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function New_Block_Is_Data( New_Block_Type )
!	Function New_Block_Is_Data returns true if the next block is a data
!	block.

	IMPLICIT NONE

!	FORMAL PARAMETERS

 	logical*1	New_Block_Is_Data
	integer*4	New_Block_Type

	New_Block_Is_Data = ( new_block_type .ne. 'FFFF'X )
	return
	end !New_Block_Is_Data

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Initialize_Data_Array( Data, Number_Of_Data_Bytes )

	IMPLICIT NONE

!	CONSTANTS

	byte		zero_byte
	PARAMETER	( ZERO_BYTE = 0 )

!	FORMAL PARAMETERS

	integer*2	Number_Of_Data_Bytes
	byte		data( Number_Of_Data_Bytes )

!	LOCAL VARIABLES

	integer*4	i

		do i=1, Number_Of_Data_Bytes
		   data(i)=zero_byte
		end do
	 end !Initialize_Data_Array

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Assign_First_128_Data_Bytes( Number_Of_Bytes_In_Record,
     :						Number_Of_Data_Bytes,
     :						Data, Line, Last_Byte,
     :						Num_Data_Rec )
!	Subroutine Assign_First_128_Data_Bytes assigns the first 128 bytes
!	(first record) of the new data block to the Data array.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*2	Number_Of_Data_Bytes
	byte		Data( Number_Of_Data_Bytes )
	byte		Line( Number_Of_Bytes_In_Record )
	integer*4	Last_Byte
	integer*4	Num_Data_Rec

!	LOCAL VARIABLES

	integer*4	Index

		do Index = 1, Number_Of_Bytes_In_Record
		   data( Index ) = line( Index )
		end do
  	        last_byte = Number_Of_Bytes_In_Record
	end !Assign_First_128_Data_Bytes


	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Get_Data_Record_Number( Number_Of_Bytes_In_Record,
     :					   Num_Data_Rec, Line )
!	Subroutine Get_Data_Record_Number gets the the current record
!	number with respect to the new data block.  The data record number is
!	contained in the first two bytes of each record of each data block.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*4	Num_Data_Rec
	byte		Line( Number_Of_Bytes_In_Record )

!	FUNCTION DECLARATIONS

	integer*4	ick2

	        num_data_rec = ick2(line(1))
	end !Get_Data_Record_Number

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Exit_Routine( Number_Of_Bytes_In_Record,
     :				 Last_Num_Of_Bytes_Processed,
     :				 Input_File_Unit_No, iunit,
     :				 ASCII_Output, ASCII_Unit_No,
     :				 Exit_Subroutine_Name,
     :				 First_Ensemble_Processed, Ensemble_Number,
     :				 Ensemble_Counter, First_Ensemble_Time, id, rd,
     :				 First_Ensemble_Year, Ensemble_Year )
!	Subroutine Exit_Routine outputs a message indicating whether the
!	program is exiting under error conditions or not.  If the program
!	is exiting under error conditions, then the name of the subroutine
!	that called Exit_Routine is output.  The range of ensembles processed
!	is given under any circumstance, as is the range of ensemble times
!	and dates that were covered.  Subroutine Exit_Routine then closes
!	any opened files, and then exits.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2	Number_Of_Bytes_In_Record
	integer*4	Last_Num_Of_Bytes_Processed
	integer*4	Input_File_Unit_No
	integer*4	iunit
	logical*1	ASCII_Output
	integer*4	ASCII_Unit_No
	character*(*)	Exit_Subroutine_Name
	integer*2	First_Ensemble_Processed
	integer*2	Ensemble_Number
	integer*4	Ensemble_Counter
	real*4		First_Ensemble_Time
	integer*2	id
	real*4		rd
	integer*2	First_Ensemble_Year
	integer*4	Ensemble_Year

!	LOCAL VARIABLES

	logical*1	File_Copy_Was_A_Success
	character*31	ASCII_File
	logical*1	File_Is_Open

	File_Copy_Was_A_Success = ( Last_Num_Of_Bytes_Processed .EQ.
     :				    Number_Of_Bytes_In_Record )
	if ( ASCII_Output .EQ. .false. ) call Open_ASCII_File( ASCII_File,
     :							  ASCII_Unit_No,
     :							  .true. )
	write( ASCII_Unit_No, * ) ( ' ' )
	write( ASCII_Unit_No, * ) ( ' ' )
	write( ASCII_Unit_No, * ) ( ' ' )
	if ( File_Copy_Was_A_Success ) then
	   write( ASCII_Unit_No, * ) ( 'FILE COPY WAS SUCCESSFUL' )
	else
 	   write( ASCII_Unit_No, * ) ( 'ERROR:  TERMINATING PROGRAM',
     :				       ' AT SUBROUTINE ', Exit_Subroutine_Name )
	end if
	write( ASCII_Unit_No, * ) ( ' ' )
	if ( Ensemble_Counter .Eq. 0 ) then
	   write( ASCII_Unit_No, * ) ( 'NO ENSEMBLES PROCESSED' )
	else
	   write( ASCII_Unit_No, * )( 'FIRST ENSEMBLE PROCESSED:  ',
     :		  First_Ensemble_Processed )
	   write( ASCII_Unit_No, * )( 'LAST ENSEMBLE PROCESSED :   ',
     :		  Ensemble_Number )
c
	   write( 47, 1116 ) First_Ensemble_Processed , First_ensemble_time,
     :        Ensemble_Number,id+rd
1116	format(i8,f10.4,i8,f10.4)
c
	   write( ASCII_Unit_No, * ) ( 'FROM : ', First_Ensemble_Time,
     :				       First_Ensemble_Year )
	   write( ASCII_Unit_No, * ) ( 'TO   : ', id + rd, Ensemble_Year )
	end if
	inquire ( UNIT = Input_File_Unit_No, OPENED = File_Is_Open )
	if ( File_Is_Open ) close( Input_File_Unit_No )
	inquire ( UNIT = iunit, OPENED = File_Is_Open )
	if ( File_Is_Open ) close( iunit )
	inquire ( UNIT = iunit, OPENED = File_Is_Open )
	if ( File_Is_Open ) close( ASCII_Unit_No )
	call EXIT
	end !Exit_Routine

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine Open_ASCII_File( ASCII_File_Name, ASCII_Unit_No,
     :				    Output_To_Screen )
!	Subroutine Open_ASCII_File opens a formatted file going by the name
!	contained in ASCII_File if the Output_To_Screen flag was raised.  If
!	not, then the standard output is opened.

	IMPLICIT NONE

!	FORMAL PARAMETERS

	character*(*)  ASCII_File_Name
	integer*4	ASCII_Unit_No
	logical*1	Output_To_Screen

!	LOCAL VARIABLES

	character*80	ASCII_File_Symbol
	integer*4	ASCII_File_Symbol_Length

	if ( Output_To_Screen ) then
	   ASCII_File_Name = 'SYS$OUTPUT'
	else
	   type *, ' '
	   type *, ' '
	   type *, 'Please input ASCII file name.'
	   read ( *, '(a)' )  ( ASCII_File_Symbol )
	   call STR$UPCASE( ASCII_File_Symbol, ASCII_File_Symbol )
	   call LIB$GET_SYMBOL( ASCII_File_Symbol, ASCII_File_Name,
     :				ASCII_File_Symbol_Length  )
	if ( ASCII_File_Symbol_Length .EQ. 0 )
     :	   ASCII_File_Name = ASCII_File_Symbol
	end if
	   open( UNIT = ASCII_Unit_No, FILE = ASCII_File_Name, STATUS = 'NEW',
     :		 FORM = 'FORMATTED' )
	end !Open_ASCII_File

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
c
	function  bck1(barg)

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*4  bck1
	byte barg
c
	if(barg.lt.0)then
		 bck1=256+barg
	else
		 bck1=barg
	end if
c
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function ick2(iarg)

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2 iarg
	integer*4 ick2

!	LOCAL VARIABLES

	integer*2 ibout
c
	call mvbits(iarg,0,8,ibout,8)
        call mvbits(iarg,8,8,ibout,0)
	if(ibout.lt.0)then
		ick2=65536+ibout
	else
		ick2=ibout
	end if
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function ick2_sign(iarg)

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*2 iarg
	integer*4 ick2_sign

!	LOCAL VARIABLES

	integer*2 ibout

        call mvbits(iarg,0,8,ibout,8)
        call mvbits(iarg,8,8,ibout,0)
	ick2_sign=ibout
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function ick4_sign(iarg)

	IMPLICIT NONE

!	FORMAL PARAMETERS

	integer*4 iarg
	integer*4 ick4_sign

!	LOCAL VARIABLES

	integer*4 ibout

        call mvbits(iarg,0,8,ibout,24)
        call mvbits(iarg,24,8,ibout,0)
        call mvbits(iarg,8,8,ibout,16)
        call mvbits(iarg,16,8,ibout,8)
	ick4_sign=ibout
	return
	end
c
	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function rck8(r8bytes)

	IMPLICIT NONE

!	CONSTANTS

	integer*2	NUMBER_OF_BYTES
	PARAMETER	( NUMBER_OF_BYTES = 8 )

!	FORMAL PARAMETERS

	byte r8bytes( NUMBER_OF_BYTES )
	real*8 rck8

!	LOCAL VARIABLES

	byte		outbytes( NUMBER_OF_BYTES )
	real*8		temp
	integer*2	J
	integer*2	K

	equivalence (outbytes,temp)
c
	if(r8bytes(8).lt.-126)then
		rck8=0.0
		return
	end if
c
	   do k=1,4
	      j=5-k
	      outbytes(2*j)=r8bytes(2*k)
	      outbytes(2*j-1)=r8bytes(2*k-1)
	   end do
	   rck8 = temp * 4.0
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function rck4(r4bytes)

	IMPLICIT NONE

!	CONSTANTS

	integer*2	NUMBER_OF_BYTES
	PARAMETER	( NUMBER_OF_BYTES = 4 )

!	FORMAL PARAMETERS

	byte r4bytes( NUMBER_OF_BYTES )
	real*4 rck4

!	LOCAL VARIABLES

	byte		outbytes( NUMBER_OF_BYTES )
	real*4		temp
	integer*2	J
	integer*2	K

	equivalence (outbytes,temp)
c
	      outbytes(1)=r4bytes(3)
	      outbytes(3)=r4bytes(1)
	      outbytes(4)=r4bytes(2)
		outbytes(2)=r4bytes(4)-1
c
	   rck4 = temp
c
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine rdi_sound(t,s,c)

	IMPLICIT NONE

!	CONSTANTS

	real*4 prs
	PARAMETER	( prs =  6. )
	real*4 sal
	PARAMETER	( sal = 35. )

!	FORMAL PARAMETERS

	real*8 t,s
	real*4 c

!	LOCAL VARIABLES

	real*4 svel,temp

	temp = t
	c    = svel(prs,temp,sal)
	WRITE(33,*)T,C
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	function grejul(iyear,imonth,iday)

	IMPLICIT NONE

!	PARAMETER VARIABLES

	integer*4 iyear,imonth,iday
	real*4 grejul

!	LOCAL VARIABLES

	integer*4 days(12,2)
	integer*4	I

	data days/0,31,59,90,120,151,181,212,243,273,304,
     *  334,0,31,60,91,121,152,182,213,244,274,305,335/
	
c	assume not a leap year
	i=1
c	check for a leap year
	if(mod(iyear,4).eq.0)i=2
c	compute julian date
	grejul=iday + days(imonth,i)
	return
	end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
      function svel(pr,t,sal)
c
c wilson oct sound speed (m/sec) jasa,1960,32,(10),1357
c

	IMPLICIT NONE

!	FORMAL PARAMETERS

	real*4 pr
	real*4 t
	real*4 sal
	real*4 svel

!	LOCAL VARIABLES

	real*4		P
	real*4		SD
	real*4		A
	real*4		V0
	real*4		V1

      p=0.1019716*(pr+10.1325)
      sd=sal-35.
   10 a=(((7.9851e-6*t-2.6045e-4)*t-4.4532e-2)*t+4.5721)*t
     x +1449.14
      svel=(7.7711e-7*t-1.1244e-2)*t+1.39799
      v0=(1.69202e-3*sd+svel)*sd+a
      a=((4.5283e-8*t+7.4812e-6)*t-1.8607e-4)*t+0.160272
      svel=(1.579e-9*t+3.158e-8)*t+7.7016e-5
      v1=svel*sd+a
      a=(1.8563e-9*t-2.5294e-7)*t+1.0268e-5
      svel=-1.2943e-7*sd+a
      a=-1.9646e-10*t+3.5216e-9
      svel=(((-3.3603e-12*p+a)*p+svel)*p+v1)*p+v0
      return
      end

	OPTIONS /EXTEND_SOURCE /G_FLOATING /CHECK=ALL
	subroutine output_init

	IMPLICIT NONE
c
	integer*4  Ensemble_Year 
	INTEGER*2  id , Number_Of_Bins, Percent_Good_Threshold
c
	integer*2  Ensemble_Number , Samples_Per_Average
c
	real*4  rd , rw , rx , rb , rr , rp , rh , rs , rf , rl , ro 
c
	real*4  rv , rat , std_dev_temp , rah , std_dev_heading , rap
c
	real*4  re , rn , rz, std_dev_pitch , rar , Std_Dev_Roll
c
	logical*1  data_flag(0:24) 
c
	real*4  vx(128) , vy(128) , vz(128) , ve(128) , amp(128) , pctgd(128)
c
	real*4  avsw(128) , rdpv(128,4) , ramp(128,4) , rspw(128,4) 
c
	real*4  n3b(128) , bmg(4) , stdv(128,4)

	integer*2	I
	integer*2	J
c
	common /output/	 Ensemble_Year , id , Number_Of_Bins ,
     :	Percent_Good_Threshold , Ensemble_Number , Samples_Per_Average , 
     :  rd , rw , rx , rb , rr , rp , rh , rs , rf , rl , ro ,
     :  rv , rat , std_dev_temp , rah , std_dev_heading , rap , std_dev_pitch,
     :	rar , Std_Dev_Roll ,
     :  re , rn , rz , data_flag , vx , vy , vz , ve , amp , pctgd,
     :  avsw , rdpv , ramp , rspw , n3b , bmg , stdv
c
	real*4		undf
	PARAMETER	( undf = 999.999 )
	integer*4	nundf
	PARAMETER	( nundf = 9999 ) 
c
	do i=0,24
		data_flag(i) = .false.
	end do
c
	Ensemble_Year = nundf
        id = nundf
        Number_Of_Bins = nundf
        Percent_Good_Threshold = nundf 
        Ensemble_Number = nundf 
        Samples_Per_Average = nundf
c
        rd = undf 
	rw = undf
        rx = undf 
        rb = undf 
 	rr = undf 
	rp = undf 
 	rh = undf 
	rs = undf 
	rf = undf 
 	rl = undf 
	ro = undf
	rv = undf 
	rat = undf 
	std_dev_temp = undf 
	rah = undf 
	std_dev_heading = undf 
	rap = undf 
	std_dev_pitch = undf 
	rar = undf 
	Std_Dev_Roll = undf
     	re = undf 
	rn = undf 
	rz = undf
c
	do i = 1,128
		vx(i)=undf
		vy(i)=undf
		vz(i)=undf
		ve(i)=undf
		amp(i)=undf
		pctgd(i)=undf
		avsw(i)=undf
		n3b(i)=undf
			do j=1,4
				rdpv(i,j)=undf
				ramp(i,j)=undf
				rspw(i,j)=undf
				stdv(i,j)=undf
			end do
	end do
c
	do i=1,4
		bmg(i)=undf
	end do
c
	return
	end
